/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import java.awt.Graphics;
import java.awt.Rectangle;

import fb.battlefield.Sprite;
import fb.battlefieldelements.Projectile;
import fb.battlefieldelements.SignWithValue;
import fb.battlefieldelements.Warrior;
import fb.util.D100;
import fb.util.FBColors;
import fb.util.GraphicTools;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * a uit with projectile weapons
 * 
 * @author farfpeuf
 * 
 */
public class Shootingunit extends Unit {

	/**
	 * additionnal characteristics
	 */
	protected int projectilesNumber, shortRange, midRange, longRange,
			shootingRows;

	/**
	 * number of volleys shot each turn
	 */
	protected int volleys = 1;

	/**
	 * shooting target
	 */
	protected Entity target;

	/**
	 * additionnal sign to display ammunitions
	 */
	protected SignWithValue arrowSign;

	/**
	 * create unit from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public Shootingunit(String ligne, int linenumber) {

		super(ligne, linenumber);

		projectilesNumber = 8;

		if (distanceWeapon == 'b') {
			shortRange = 6;
			longRange = 18;
			shootingRows = 4;
		} else if (distanceWeapon == 'l') {
			shortRange = 8;
			longRange = 30;
			shootingRows = 4;
		} else if (distanceWeapon == 'x') {
			shortRange = 8;
			longRange = 30;
			shootingRows = 2;
		}
		midRange = shortRange + longRange / 2;

	}

	/**
	 * get entity weapon as a string
	 * 
	 * @return entity weapon as a string
	 */
	public String weaponString() {
		return distanceWeaponString() + "/" + super.weaponString();
	}

	/**
	 * string for distance weapon
	 * 
	 * @return string for distance weapon
	 */
	public String distanceWeaponString() {
		if (distanceWeapon == 'b') {
			return Resources.getString("bow");
		} else if (distanceWeapon == 'l') {
			return Resources.getString("longBow");
		} else if (distanceWeapon == 'x') {
			return Resources.getString("crossbow");
		} else
			return "-";
	}

	/**
	 * initialize unit at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		arrowSign = new SignWithValue(this, projectilesNumber);
		arrowSign.setImage(Resources.getSign("arrowcounter"));
		signs.add(0, arrowSign);

	}

	/**
	 * reset unit after end of battle
	 * 
	 * @param winner
	 *            did the unit army won the battle
	 */
	public void endBattle(boolean winner) {
		super.endBattle(winner);
		arrowSign = null;
		projectilesNumber = 8;
	}

	/**
	 * get bounds for shooting range shape
	 * 
	 * @return bounds for shooting range shape
	 */
	public Rectangle getShootingRangeBounds() {
		int rx = (int) (getX() - getLongRange() * 8);
		int ry = (int) (getY() - getLongRange() * 8);
		int rw = (int) (getLongRange() * 16);
		int rh = (int) (getLongRange() * 16);
		return new Rectangle(rx, ry, rw, rh);
	}

	/**
	 * paint shooting range
	 * 
	 * @param g
	 *            graphics
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 * @param scale
	 *            view scale
	 */
	public void paintShootingRange(Graphics g, int dx, int dy, int scale) {
		int x, y;
		int rayon = getShortRange() * 8;
		x = getX();
		y = getY();
		g.setColor(FBColors.secondary);
		GraphicTools.drawScaledCircle(g, dx, dy, scale, rayon, x, y);
		g.setColor(FBColors.primary);
		rayon = (getShortRange() + getLongRange()) * 4;
		GraphicTools.drawScaledCircle(g, dx, dy, scale, rayon, x, y);
		rayon = getLongRange() * 8;
		g.setColor(FBColors.errorColor);
		GraphicTools.drawScaledCircle(g, dx, dy, scale, rayon, x, y);
	}

	/**
	 * get number of frames in entity sprite
	 */
	public int getNumberOfFrames() {
		return super.getNumberOfFrames() + 32;
	}

	/**
	 * add sequences to sprite
	 */
	public void addSpriteSequences(Sprite sprite) {
		super.addSpriteSequences(sprite);
		addSpriteShootingSequences(sprite);
	}

	/**
	 * add shooting sequences to sprite
	 */
	public void addSpriteShootingSequences(Sprite sprite) {
		sprite.addSpritsequence("shoot0", 124, 132, 140, 148);
		sprite.addSpritsequence("shoot1", 125, 133, 141, 149);
		sprite.addSpritsequence("shoot2", 126, 134, 142, 150);
		sprite.addSpritsequence("shoot3", 127, 135, 143, 151);
		sprite.addSpritsequence("shoot4", 120, 128, 136, 144);
		sprite.addSpritsequence("shoot5", 121, 129, 137, 145);
		sprite.addSpritsequence("shoot6", 122, 130, 138, 146);
		sprite.addSpritsequence("shoot7", 123, 131, 139, 147);
	}

	/**
	 * is this entity currently shooting
	 * 
	 * @return true if this entity currently shooting
	 */
	public boolean isShooting() {
		for (int i = 0; i < getFront() * getShootingRows() && i < getNumber(); i++) {
			if (getModel(i).getAction() == 's') {
				return true;
			}
		}
		return false;
	}

	/**
	 * shoot
	 * 
	 * @param target
	 *            target
	 */
	public void shoot(Entity target) {

		// --hits
		int hits = 0, tohit = shootToHit(target), shootingworth = shootingWorth();

		for (int i = 0; i < shootingworth; i++) {
			int d100 = D100.roll();
			if (d100 >= D100.LUCKY) {
				hits++;
			} else if (d100 < D100.FUMBLE) {
			} else if (d100 >= tohit) {
				hits++;
			}
		}

		// --kills
		int kills = 0, tokill = shootToKill(target);

		for (int i = 0; i < hits; i++) {
			int d100 = D100.roll();
			if (d100 >= D100.LUCKY) {
				kills++;
			} else if (d100 < D100.FUMBLE) {
			} else if (d100 >= tokill) {
				kills++;
			}
		}

		// tir sur des figurines isolees

		if (target.surface() < 3) {
			kills = (int) (kills * target.surface() / 3);
		}

		target.wound(kills);

	}

	/**
	 * consume volleys used for shooting
	 * 
	 */
	public void spendVolleys() {
		projectilesNumber -= volleys;
		arrowSign.setvalue(projectilesNumber);
	}

	/**
	 * calculate shooting worth
	 * 
	 * @return shooting worth
	 */
	public int shootingWorth() {
		return getWorth() * getNumber() * volleys;

	}

	/**
	 * calculate shooting strength
	 * 
	 * @return shooting strength
	 */
	public int shootingStrength() {
		return getStrength();
	}

	/**
	 * calculate score needed to hit target
	 * 
	 * @param target
	 *            target
	 * @return score needed to hit target
	 */
	public int shootToHit(Entity target) {
		int tohit = 50;

		if (level == 2) {
			tohit -= 15;
		} else if (level == 3) {
			tohit -= 30;
		} else {
			tohit += 15;
		}
		// very short range

		int dist = lineOfSightTo(target);

		if (dist < shortRange * 8 && !isFlying() && !target.isFlying()) {
			tohit += -15;
		}
		if (dist > midRange * 8 || isFlying() || target.isFlying()) {
			tohit += 15;
		}
		if (volleys > 1) {
			tohit += 15;
		}
		if (suffersBadLight()) {
			tohit += 15;
		}
		if (isDisorganized()) {
			tohit += 15;
		}
		if (getState() == 's') {
			tohit += 15;
		} else if (getState() == 'b') {
			tohit += 15;
		}

		return tohit;
	}

	/**
	 * score needed to kill target
	 * 
	 * @param shooterWeapon
	 *            shooter weapon
	 * @param targetArmor
	 *            target armor
	 * @return score needed to kill target
	 */
	public int shootToKill(char shooterWeapon, char targetArmor) {
		if (shooterWeapon == 'b') {
			if (targetArmor == 'x') {
				return 6;
			} else if (targetArmor == 'h') {
				return 6;
			} else if (targetArmor == 'm') {
				return 5;
			} else if (targetArmor == 'l') {
				return 4;
			} else {
				return 3;
			}
		} else if (shooterWeapon == 'l') {
			if (targetArmor == 'x') {
				return 6;
			} else if (targetArmor == 'h') {
				return 5;
			} else if (targetArmor == 'm') {
				return 4;
			} else if (targetArmor == 'l') {
				return 3;
			} else {
				return 2;
			}
		} else if (shooterWeapon == 'h') {
			if (targetArmor == 'x') {
				return 4;
			} else if (targetArmor == 'h') {
				return 3;
			} else if (targetArmor == 'm') {
				return 3;
			} else {
				return 2;
			}
		} else {
			if (targetArmor == 'x') {
				return 5;
			} else if (targetArmor == 'h') {
				return 4;
			} else if (targetArmor == 'm') {
				return 3;
			} else {
				return 2;
			}
		}
	}

	/**
	 * score needed to kill target
	 * 
	 * @param target
	 *            target
	 * @return score needed to kill target
	 */
	public int shootToKill(Entity target) {
		int tokill = shootToKill(distanceWeapon, target.armor);

		tokill -= (shootingStrength() * 15);
		tokill += (target.getResistance() * 15);

		if (target.hasShield) {
			tokill += 15;
		}
		return tokill;
	}

	/**
	 * @return number of projectiles shot
	 */
	protected int getNbProjectiles() {
		return 1;
	}

	/**
	 * @return projectile type
	 */
	public char getProjectileType() {
		return 'a';
	}

	/**
	 * start shooting animation
	 * 
	 * @param target
	 *            shooting target
	 */
	public void animShoot(Entity target) {
		double angledeshoot = angleTo(target);

		Warrior targetmodel, model;
		int nbProjectiles = getNbProjectiles();

		for (int i = 0; i < getNumber(); i++) {
			model = getModel(i);
			model.animshoot(angledeshoot);
			for (int j = 0; j < nbProjectiles; j++) {
				if (target.isIndividual()) {
					targetmodel = ((Individual) target).getModel();
				} else {
					targetmodel = ((Unit) target).nextTarget();
				}
				model.prepareProjectile(new Projectile(model.getX(), model
						.getY(), targetmodel, getProjectileType()));
			}
		}
	}

	/**
	 * returns unit as a string to save it to a file
	 * 
	 * @return unit as a string
	 */
	public String saveString() {
		return super.saveString() + '\t' + projectilesNumber;

	}

	/**
	 * load unit from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		projectilesNumber = StringTools.readBufferInt(buffy);
	}

	/**
	 * @return resource key
	 */
	protected String getResourceKey() {
		return "shootingunit";
	}

	/**
	 * @return Returns the shootingRows.
	 */
	public int getShootingRows() {
		return shootingRows;
	}

	/**
	 * @return Returns the longRange.
	 */
	public int getLongRange() {
		return longRange;
	}

	/**
	 * @param longRange
	 *            The longRange to set.
	 */
	public void setLongRange(int longRange) {
		this.longRange = longRange;
	}

	/**
	 * @return Returns the midRange.
	 */
	public int getMidRange() {
		return midRange;
	}

	/**
	 * @param midRange
	 *            The midRange to set.
	 */
	public void setMidRange(int midRange) {
		this.midRange = midRange;
	}

	/**
	 * @return Returns the shortRange.
	 */
	public int getShortRange() {
		return shortRange;
	}

	/**
	 * @param shortRange
	 *            The shortRange to set.
	 */
	public void setShortRange(int shortRange) {
		this.shortRange = shortRange;
	}

	/**
	 * @return Returns the target.
	 */
	public Entity getTarget() {
		return target;
	}

	/**
	 * @param target
	 *            The target to set.
	 */
	public void setTarget(Entity target) {
		this.target = target;
	}

	/**
	 * @return Returns the volleys.
	 */
	public int getVolleys() {
		return volleys;
	}

	/**
	 * @param volleys
	 *            The volleys to set.
	 */
	public void setVolleys(int volleys) {
		this.volleys = volleys;
	}

	/**
	 * @return Returns the projectilesNumber.
	 */
	public int getProjectilesNumber() {
		return projectilesNumber;
	}

}
