/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;
import java.awt.Rectangle;
import java.awt.geom.Area;

import fb.battlefield.BattleFieldModel;
import fb.battlefieldelements.Sign;
import fb.main.Catalog;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.util.StringTools;
/**
 * a creature summoned by a spell caster
 * @author farfpeuf
 *  
 */
public abstract class SummonedCreature extends Individual {
	/**
	 * the summoner
	 */
	private SpellCaster summoner;
	/**
	 * the summoner id
	 */
	private int summonerId;
	/**
	 * is this creature under control ?
	 */
	private boolean underControl;
	/**
	 * calculate cost
	 */
	public abstract int getCost();
	/**
	 * create inidividual from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public SummonedCreature(String ligne, int linenumber) {
		super(ligne, linenumber);
	}
	/**
	 * summon a creature
	 * @param master summoner
	 * @param linenumber catalog line number
	 */
	public static void summon(SpellCaster master, int linenumber) {
		SummonedCreature summoned =
			(SummonedCreature) ((Entity) Catalog.getFullCatalog().get(linenumber))
				.clone();
		summoned.summonerId = master.getId();
		summoned.underControl = true;
		summoned.initLifePoints(summoned.initialPdv);
		summoned.initBattle();
		summoned.create();
		master.command.addEntity(summoned);

		summoned.setInitialPosition();
		summoned.setScale(0f);
		Battle.getInstance().getActiveEntities().add(summoned);
		Battle.getInstance().getAllEntities().add(summoned);

		while (summoned.getScale() < 1) {
			try {
				summoned.setScale(summoned.getScale() + 0.1f);
				Thread.sleep(125);
			} catch (InterruptedException e) {
				System.out.println("interrupted");
			}
		}

		summoned.setScale(1f);
		summoned.model.resetAreas();
	}
/**
 * bannish this creature
 *
 */
	public void bannish() {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.BANNISH + getId());
		} else {
			doBannish();
		}
	}
/**
 * start banninshing animation
 *
 */
	private void animBannish() {
		while (this.getScale() > 0.1f) {
			try {
				setScale(this.getScale() - 0.1f);
				Thread.sleep(125);
			} catch (InterruptedException e) {
				System.out.println("interrupted");
			}
		}
	}
/**
 * do bannish this creature
 *
 */
	public void doBannish() {
		command.removeActiveEntity(this);
		clearContacts();
		Battle.getInstance().getActiveEntities().remove(this);
		Battle.getInstance().getAllEntities().remove(this);

		for (int i = 0; i < signs.size(); i++) {
			BattleFieldModel.getInstance().removeSign((Sign) signs.get(i));
		}

		if (!FB.getInstance().isBatch()) {
			animBannish();
		}

		BattleFieldModel.getInstance().removeWarrior(model);
	}
	/**
	 * set creature initial position
	 *
	 */
	public void setInitialPosition() {
		boolean placed = false;
		int decX, decY, destX, destY;
		Area deployArea =
			new Area(
				new Rectangle(
					summoner.getX() - 100,
					summoner.getY() - 100,
					summoner.getX() + 100,
					summoner.getY() + 100));
		deployArea.subtract(getMovementObstacles());
		doRotate(summoner.getAngle());
		for (int i = 0; !placed && i < 20; i++) {
			decX = 10 * i / 2 * (-1 ^ (i % 2));
			destX = summoner.getX() + decX;
			for (int j = 0; !placed && j < 20; j++) {
				decY = 10 * j / 2 * (-1 ^ (j % 2));
				destY = summoner.getY() + decY;
				if (canDeployTo(destX, destY, deployArea)) {
					doDeployTo(destX, destY);
					placed = true;
				}
			}
		};
	}
	/**
	 * drain magic power from summoner
	 * @return true if creature is still under control
	 */
	public boolean drainMagic() {
		if (summoner.spendMagicPoints(magicCost())) {
			return true;
		}
		if (underControl) {
			underControl = false;
			calculateContacts();
		}
		return false;
	}
	/**
	 * calculate magic cost
	 */
	public abstract int magicCost();
	/**
	 * @return the summoner
	 */
	public SpellCaster getSummoner() {
		return summoner;
	}
	/**
	 * test if another entity is an ennemy of this entity
	 * 
	 * @param e
	 *            the other entity
	 * @return true if the 2 entities are ennemies
	 */
	public boolean isEnnemy(Entity e) {
		return !underControl || super.isEnnemy(e);
	}
	/**
	 * returns individual as a string to save it to a file
	 * 
	 * @return individual as a string
	 */
	public String saveString() {
		String result = super.saveString();
		result += ("\t" + (underControl ? 'y' : 'n'));
		result += ("\t" + summoner.getId());
		return result;
	}
	/**
	 * load individual from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		underControl =
			(StringTools.readBufferChar(buffy) == 'y') ? true : false;
		summonerId = StringTools.readBufferInt(buffy);

	}
	/**
	 * @return true if creature is under Control
	 */
	public boolean isUnderControl() {
		return underControl;
	}

	/**
	 * initialize entity at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		SpellCaster master =
			(SpellCaster) Battle.getInstance().getAllEntities().get(summonerId);
		this.summoner = master;
	}



}