/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fb.battlefield.BattleFieldModel;
import fb.battlefieldelements.Sign;
import fb.battlefieldelements.Warrior;
import fb.main.Battle;
import fb.main.FB;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.phases.CombatPhase;
import fb.sound.SoundPlayer;
import fb.util.D100;
import fb.util.DX;
import fb.util.FBColors;
import fb.util.GeometryTools;
import fb.util.Resources;
import fb.util.StringTools;
import fb.util.UnitTools;

/**
 * a group of warriors
 * 
 * @author farfpeuf
 * 
 */
public class Unit extends Entity {

	// property change constants

	public static final String ORDER = "order";

	public static final String MORALE = "morale";

	public static final String NUMBER = "number";

	public static final String DISORGANIZED = "disorganized";

	// used to hold warrios life points during unit creation
	private int[] lifePointsHolder;

	/**
	 * iis the unit disorganized
	 */
	private boolean disorganized = false;

	/**
	 * coordinates
	 */
	private int x = 0, y = 0;

	/**
	 * casualties made to oppenents
	 */
	private int opponentsCasualties = 0;

	/**
	 * number of rows
	 */
	protected int nbRows = 1;

	/**
	 * number of warrios on the front line
	 */
	private int front = 0;

	/**
	 * additionnal characteristics
	 */
	private int number = 1, morale = 7, musicians = 0, initialNumber = 1;

	/**
	 * current order
	 */
	private char order = 'o';

	/**
	 * percentage of casualties endured this turn
	 */
	private float casualtyPercentage;

	/**
	 * unit orientation angle
	 */
	private double orientation = 0.0;

	/**
	 * list of dead warriors
	 */
	private List deads;

	/**
	 * additionnal signs
	 */
	private Sign ordersign, statesign, organizationsign;

	/**
	 * did the unit changed its orders this turn
	 */
	private boolean changedOrders = false;

	/**
	 * additionnal characteristics
	 */
	protected boolean twoRows = false, hasLostStandard = false,
			closeopposingtc = false, closefriendlytc = false, hasChief = true,
			hasChampion = false, hasStandardBearer = false, upgraded = false,
			casualtyThisTurn = false, terribleopponent = false,
			surrounding = false;

	/**
	 * next casulaty index
	 */
	private int nextIndex;

	/**
	 * create unit from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public Unit(String ligne, int linenumber) {
		super(ligne, linenumber);
		initNumber();
	}

	/**
	 * default number in army editor
	 * 
	 */
	protected void initNumber() {
		number = getMin() + getMax() / 2;
	}

	/**
	 * reset unit after end of battle
	 * 
	 * @param winner
	 *            did the unit army won the battle
	 */
	public void endBattle(boolean winner) {
		if (getState() != 'r') {
			setState('n');
			if (winner) {
				morale = 8;
			} else {
				morale = 6;
			}
			initialNumber = number;
		} else {
			morale = 4;
			setState('s');
			if (winner) {
				initialNumber = Math.max((int) (number * 0.7), 1);
			} else {
				initialNumber = (int) (number * 0.4);
			}
			number = initialNumber;
		}
		lifePointsHolder = new int[number];
		Warrior figure;

		if (models != null) {
			for (int i = 0; i < number; i++) {
				figure = (Warrior) models.get(i);
				lifePointsHolder[i] = figure.getLifePoints();
			}
		}

		order = 'o';
		ordersign = null;
		statesign = null;
		organizationsign = null;
		deads = null;
		front = 0;
		vertex = null;
		area = null;
		twoRows = false;
		hasLostStandard = false;
		surrounding = false;
		orientation = 0.0;

		super.endBattle(winner);
	}

	/**
	 * get this unit movement points
	 * 
	 * @return this unit movement points
	 */
	public int getMovement() {
		int movement = super.getMovement();
		if (order == 'a') {
			movement += 2;
		}
		return movement;
	}

	/**
	 * initialize unit at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		deads = new ArrayList();
		ordersign = new Sign(this);
		statesign = new Sign(this);
		organizationsign = new Sign(this);
		signs.add(0, ordersign);
		signs.add(statesign);
		signs.add(organizationsign);
		if (isDisorganized()) {
			organizationsign.setImage(Resources.getSign("disorganized"));
		}
		if (getState() == 's') {
			statesign.setImage(Resources.getSign("shaken"));
		} else if (getState() == 'b') {
			statesign.setImage(Resources.getSign("bloodlust"));
		}
		if (front == 0) {
			initFront();
		}
		vertex = UnitTools.calculateVertex(this);
		area = UnitTools.vertexToArea(vertex);

	}

	/**
	 * initialize front line at the beginning of a battle
	 * 
	 */
	public void initFront() {
		front = (int) Math.sqrt(number) + 1;
		if (front > number) {
			front = number;
		}
	}

	/**
	 * change orders
	 * 
	 * @param neworder
	 *            the new orders
	 */
	public void changeOrder(char neworder) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.NEWUNITORDER + getId() + "#"
							+ Character.toString(neworder));
		} else {
			doChangeOrder(neworder);
		}
	}

	/**
	 * do change orders
	 * 
	 * @param neworder
	 *            the new orders
	 */
	public void doChangeOrder(char neworder) {

		setOrder(neworder);
		changedOrders = true;
		if (order == 'a') {
			ordersign.setImage(Resources.getSign("attack"));
		} else if (order == 'o') {
			ordersign.setImage(Resources.getSign("oppose"));
		} else {
			ordersign.setImage(Resources.getSign("hold"));
		}

	}

	/**
	 * set orders
	 * 
	 * @param neworder
	 *            the new orders
	 */
	public void setOrder(char neworder) {

		char oldValue = this.order;
		this.order = neworder;
		firePropertyChange(ORDER, oldValue, order);

	}

	/**
	 * calculate unit cost
	 */
	public int getCost() {
		int cost = initialNumber * price + musicians * 5;
		if (upgraded) {
			cost += (initialNumber * upgradePrice);
		}
		if (hasChief) {
			cost += 5;
		}
		if (hasStandardBearer) {
			cost += 5;
		}
		if (hasChampion) {
			cost += 5;
		}
		return cost;
	}

	/**
	 * create all unit warriors
	 * 
	 */
	public void createModel() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = new Warrior(this,
					(lifePointsHolder != null) ? lifePointsHolder[i]
							: initialPdv);
			model.init();
			models.add(model);
			BattleFieldModel.getInstance().addWarrior(model);
		}
		if (order == 'a') {
			ordersign.setImage(Resources.getSign("attack"));
		} else if (order == 'o') {
			ordersign.setImage(Resources.getSign("oppose"));
		} else {
			ordersign.setImage(Resources.getSign("hold"));
		}
	}

	/**
	 * calculate base surface
	 * 
	 * @return base surface
	 */
	public float surface() {
		return getWidth() * getLength() * number;
	}

	/**
	 * get unit total life points
	 * 
	 * @return unit total life points
	 */
	public int getTotalLifePoints() {
		int result = 0;
		for (int i = 0; i < number; i++) {
			result += ((Warrior) models.get(i)).getLifePoints();
		}
		return result;
	}

	/**
	 * get initial life points
	 * 
	 * @return initial life points
	 */
	public int getInitialTotalLifePoints() {
		return initialPdv * initialNumber;
	}

	/**
	 * get Border
	 * 
	 * @return Border
	 */
	public Shape getBorder() {
		return area;
	}

	/**
	 * get front line width
	 * 
	 * @return front line width
	 */
	public int getFrontWidth() {
		return (int) (getWidth() * front * 8);
	}

	/**
	 * get max diagonal
	 * 
	 * @return maximum diagonal
	 */
	public int getMaxDiag() {
		float pythagore = getWidth() * getWidth() / 4f * front * front
				+ getLength() * getLength() * nbRows * nbRows;
		return (int) Math.ceil(Math.sqrt(pythagore));
	}

	/**
	 * get x current coordinate
	 * 
	 * @return x current coordinate
	 */
	public int getX() {
		return x;
	}

	/**
	 * get y current coordinate
	 * 
	 * @return y current coordinate
	 */
	public int getY() {
		return y;
	}

	/**
	 * get Angle
	 * 
	 * @return Angle
	 */
	public double getAngle() {
		return orientation;
	}

	/**
	 * change angle
	 * 
	 * @param angle
	 *            new angle
	 */
	public void setAngle(double angle) {
		orientation = angle;
	}

	/**
	 * get Contact Area
	 * 
	 * @return Contact Area
	 */
	public Area getContactArea() {
		Area result = new Area();
		for (int i = 0; i < number; i++) {
			result.add(getModel(i).getContactArea());
		}
		return result;
	}

	/**
	 * get LeftMost Vertex
	 * 
	 * @return LeftMost Vertex
	 */
	public Point getLeftMostVertex() {
		return leftModel().getA();
	}

	/**
	 * get RightMost Vertex
	 * 
	 * @return RightMost Vertex
	 */
	public Point getRightMostVertex() {
		return rightModel().getB();
	}

	/**
	 * calculate movement points for a slowed move
	 * 
	 * @param mousex
	 *            destination x coordinate
	 * @param mousey
	 *            destination y coordinate
	 * @return movement points for slowed move
	 */
	public float slowedMoveTo(int mousex, int mousey) {
		if (isFlying()) {
			return 1f;
		} else {
			Area obstacles, path, inter;
			path = getPathTo(mousex, mousey);
			inter = new Area(path);
			obstacles = new Area(Battle.getInstance().getSlowObstacles());
			obstacles.subtract(this.area);
			inter.intersect(obstacles);
			int pathSurface = GeometryTools.calculateSurface(path);
			if (pathSurface == 0) {
				return 1;
			}
			int intersurface = GeometryTools.calculateSurface(inter);
			return 1f + (float) intersurface / (float) pathSurface;
		}
	}

	/**
	 * paint shape for a possible movement
	 * 
	 * @param g
	 *            graphics
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 * @param mouseX
	 *            mouse x coordinate
	 * @param mouseY
	 *            mouse y coordinate
	 */
	public void paintMovementOK(Graphics g, int distx, int disty, int destx,
			int desty, int mouseX, int mouseY) {
		FB.getInstance().setBattlefieldCursor(FB.DEFAULT_CURSOR);
		int movelength = (int) (0.99 + Math.sqrt(distx * distx + disty * disty) / 8);
		movelength = (int) ((float) movelength * slowedMoveTo(destx, desty));
		if (movelength > getRemainingMove()) {
			movelength = getRemainingMove();
			g.setColor(FBColors.warningColor);
		} else {
			g.setColor(FBColors.secondary);
		}
		g.drawString(Integer.toString(movelength), mouseX, mouseY);
	}

	/**
	 * paint movement range shape
	 * 
	 * @param g
	 *            graphics
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 * @param scale
	 *            view scale
	 */
	public void paintMovementRange(Graphics g, int dx, int dy, int scale) {
		AffineTransform at;
		int mvt = getRemainingMove() * 8;

		// Area devant=new Area(new
		// Arc2D.Double(x-mvt,y-mvt-(0.5*length*8),2*mvt,2*mvt,45,90,Arc2D.PIE));
		// Area derriere=new Area(new
		// Arc2D.Double(x-mvt,y-mvt+(((double)nbrangs-0.5)*length*8),2*mvt,2*mvt,225,90,Arc2D.PIE));
		destArea = new Area();

		if (order == 'a') {
			Area devant = new Area(new Arc2D.Double(x - mvt, y - mvt, 2 * mvt,
					2 * mvt, 45, 90, Arc2D.PIE));
			destArea.add(devant);
		} else if (order == 'o') {
			Area all = new Area(new Arc2D.Double(x - mvt, y - mvt, 2 * mvt,
					2 * mvt, 0, 360, Arc2D.PIE));
			destArea.add(all);
		}
		at = AffineTransform.getRotateInstance(-orientation, x, y);
		destArea = new Area(at.createTransformedShape(destArea));

		Area obstacles = getMovementObstacles();
		obstacles.subtract(this.area);
		destArea.subtract(obstacles);
		/*
		 * Entity entity;
		 * 
		 * for (int i=0;i <FB.getInstance().entities.size();i++){
		 * entity=(Entity)(FB.getInstance().entities.get(i));
		 * 
		 * if (entity !=this){ tmparea=new Area(entity.area);
		 * tmparea.intersect(destArea); if(!tmparea.isEmpty()){
		 * destArea.subtract(tmparea); } } }
		 * 
		 */
		Area scaledest = new Area(AffineTransform
				.getScaleInstance(scale, scale)
				.createTransformedShape(destArea));
		scaledest = new Area(AffineTransform.getTranslateInstance(dx, dy)
				.createTransformedShape(scaledest));
		((Graphics2D) g).draw(scaledest);
	}

	/**
	 * calculate distance to another entity
	 * 
	 * @param e
	 *            the other entity
	 * @return distance to other entity
	 */
	public int distanceTo(Entity e) {
		int distance = 9999;
		int tmpdist;
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) (models.get(i));
			tmpdist = e.distanceFrom(model);
			if (tmpdist < distance) {
				distance = tmpdist;
			}
		}
		return distance;
	}

	/**
	 * get translated line of movement
	 * 
	 * @param e
	 *            target entity
	 * @param tx
	 *            initial x coordinate
	 * @param ty
	 *            initial y coordinate
	 * @return line of movement length
	 */
	public int translatedLineOfMovementTo(Entity e, int tx, int ty) {
		int lomto = 9999;
		int tmplom;
		Warrior model;
		try {

			for (int i = 0; i < front; i++) {
				model = (Warrior) (models.get(i));
				tmplom = e.translatedLineOfMovementFrom(model, tx, ty);
				if (tmplom < lomto) {
					lomto = tmplom;
				}
			}
		} catch (Exception ex) {
			FB.exception(ex);
		}
		return lomto;
	}

	/**
	 * get distance from a warrior
	 * 
	 * @param originmodel
	 *            the warrior
	 * @return distance from the warrior
	 */
	public int distanceFrom(Warrior originmodel) {
		int distance = 9999;
		int tmpdist, dx, dy;
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) (models.get(i));
			dx = originmodel.getX() - model.getX();
			dy = originmodel.getY() - model.getY();
			tmpdist = (int) java.lang.Math.sqrt(dx * dx + dy * dy);
			if (tmpdist < distance) {
				distance = tmpdist;
			}
		}
		return distance;
	}

	/**
	 * get front leftmost warrior
	 * 
	 * @return front leftmost warrior
	 */
	public Warrior leftModel() {
		return getModel(0);
	}

	/**
	 * get front rightmost warrior
	 * 
	 * @return front rightmost warrior
	 */
	public Warrior rightModel() {
		return getModel(front - 1);
	}

	/**
	 * get line of sight from a warrior
	 * 
	 * @param originmodel
	 *            the warrior
	 * @return line of sigh length
	 */
	public int lineOfSightFrom(Warrior originmodel) {
		// FB.getInstance().debug("losfrom " + originmodel.entity.name);
		int x0 = originmodel.getX();
		int y0 = originmodel.getY();
		int losfrom = Integer.MAX_VALUE;
		int tmplos;
		Warrior model;

		Point losP0 = new Point(x0, y0);
		Point losP1 = new Point();
		for (int i = 0; i < front; i++) {
			model = (Warrior) (models.get(i));
			losP1.x = model.getX();
			losP1.y = model.getY();
			if (clearLineOfSight(losP0, losP1, originmodel.getEntity())) {
				int dx = x0 - model.getX();
				int dy = y0 - model.getY();
				tmplos = (int) java.lang.Math.sqrt(dx * dx + dy * dy);
			} else {
				tmplos = Integer.MAX_VALUE;
			}
			if (tmplos < losfrom) {
				losfrom = tmplos;
			}
		}
		// FB.getInstance().debug("end losfrom");
		return losfrom;
	}

	/**
	 * get translated line of movement from a warrior
	 * 
	 * @param originmodel
	 *            the warrior
	 * @param tx
	 *            initial x coordinate
	 * @param ty
	 *            initial y coordinate
	 * @return line of sigh length
	 */
	public int translatedLineOfMovementFrom(Warrior originmodel, int tx, int ty) {
		int x0 = originmodel.getX() + tx;
		int y0 = originmodel.getY() + ty;
		int lomfrom = 9999;
		int tmplom;
		Warrior model;
		int[] areax, areay;
		areax = new int[3];
		areay = new int[3];
		Polygon lom;
		for (int i = 0; i < front; i++) {
			model = (Warrior) (models.get(i));
			areax[0] = x0 - 1;
			areax[1] = x0 + 1;
			areax[2] = model.getX();
			areay[0] = y0;
			areay[1] = y0;
			areay[2] = model.getY();
			lom = new Polygon(areax, areay, 3);
			if (clearLineOfMovement(lom, originmodel.getEntity())) {
				int dx = x0 - model.getX();
				int dy = y0 - model.getY();
				tmplom = (int) java.lang.Math.sqrt(dx * dx + dy * dy);
				;
			} else {
				tmplom = 9999;
			}
			if (tmplom < lomfrom) {
				lomfrom = tmplom;
			}
		}
		return lomfrom;
	}

	/**
	 * fight
	 * 
	 */
	public void fight() {
		List weights, targets, contacts;
		Entity entity2;
		Warrior model;
		int index, nbopposing;
		twoRows = false;
		weights = new ArrayList();
		targets = new ArrayList();
		boolean possible2rows = ((weapon == 's' || weapon == 'p')
				&& !isDisorganized() && getState() == 'n');
		boolean possible3rows = (weapon == 'p' && possible2rows);
		for (int i = 0; i < number; i++) {
			model = (Warrior) (models.get(i));
			contacts = model.getEnnemyContacts();
			nbopposing = contacts.size();
			// 2e rang
			if (nbopposing == 0 && possible2rows) {
				contacts = model.getEnnemySpearContacts();
				nbopposing = contacts.size();
				twoRows = true;
			}
			// 3e rang
			if (nbopposing == 0 && possible3rows) {
				contacts = model.getEnnemyPikeContacts();
				nbopposing = contacts.size();
				twoRows = true;
			}
			for (int j = 0; j < nbopposing; j++) {
				entity2 = ((Warrior) (contacts.get(j))).getEntity();
				index = targets.indexOf(entity2);
				if (index != -1) {
					double oldweight = ((Double) (weights.get(index)))
							.doubleValue();
					Double newweight = new Double(oldweight
							+ (double) (1 / nbopposing));
					weights.set(index, newweight);
				} else {
					targets.add(entity2);
					weights.add(new Double((double) (1 / nbopposing)));
				}
			} // for2
		} // for1
		double reste, weight;
		int combatworth;
		reste = 0.0;
		Entity opponent;
		for (int i = 0; i < targets.size(); i++) {
			weight = ((Double) (weights.get(i))).doubleValue();
			combatworth = (int) (weight * getWorth() + reste + 0.01);
			reste = weight * getWorth() - combatworth;
			opponent = (Entity) targets.get(i);
			opponentsCasualties += combat(combatworth, opponent);
		}
	}

	/**
	 * calculate contacts
	 * 
	 */
	public void calculateContacts() {
		clearContacts();
		for (int i = 0; i < number; i++) {
			getModel(i).calculateContacts();
		}
	}

	/**
	 * clear contacts
	 * 
	 */
	public void clearContacts() {
		terribleopponent = false;
		friendlycontacts.clear();
		opposingContacts.clear();
		secondcontacts.clear();
		thirdcontacts.clear();
		for (int i = 0; i < number; i++) {
			getModel(i).clearcontacts();
		}
	}

	/**
	 * add a contact with another entity
	 * 
	 * @param entity2
	 *            the other entity
	 * @param nature
	 *            type of contact
	 */
	public void addContact(Entity entity2, int nature) {
		if (nature == 0) {
			friendlycontacts.add(entity2);
		} else {
			if (entity2.terrible) {
				terribleopponent = true;
			}
			if (nature == 1) {
				opposingContacts.add(entity2);
			} else if (nature == 2) {
				secondcontacts.add(entity2);
			} else {
				thirdcontacts.add(entity2);
			}
		}
	}

	/**
	 * calculate base area
	 * 
	 */
	public void calculateArea() {
		if (number > 0) {
			area = UnitTools.vertexToArea(vertex);
		}
	}

	/**
	 * calculate base vertice
	 * 
	 */
	public void calculateVertex() {
		vertex = UnitTools.calculateVertex(this);
	}

	/**
	 * calculate all the warriors positions
	 * 
	 */
	public void calculateWarriorsPositions() {
		organize();
		orientate();
		calculateVertex();
		calculateArea();
		calculateContacts();
		if (surrounding) {
			calculateSurroundPositions();
		}
		animMovement();
	}

	/**
	 * orientate all the warriors
	 * 
	 */
	public void orientate() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			model.orientate(orientation, x, y);
			model.setAngle(orientation);
		}
		for (int i = 0; i < signs.size(); i++) {
			((Sign) signs.get(i)).rotate(orientation, x, y);
		}
	}

	/**
	 * translate unit
	 * 
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 */
	public void translate(int dx, int dy) {
		x += dx;
		y += dy;
		calculateWarriorsPositions();
	}

	/**
	 * do move this unit to specified location
	 * 
	 * @param x
	 *            destination x coordinate
	 * @param y
	 *            destination y coordinate
	 * @param consumeMovement
	 *            must movement points be consumed
	 */
	public void doMoveTo(int nx, int ny, boolean consumeMovement) {
		if (consumeMovement) {
			int movelength = (int) (0.99 + Math.sqrt((nx - x) * (nx - x)
					+ (ny - y) * (ny - y)) / 8);
			movelength = (int) (movelength * slowedMoveTo(nx, ny));
			if (movelength > getRemainingMove()) {
				movelength = getRemainingMove();
				disorganize();
			}
			consumeMovement(movelength);
		}
		x = nx;
		y = ny;
		calculateWarriorsPositions();
	}

	/**
	 * do move unit to specified destination
	 * 
	 * @param x
	 *            destination x coordinate
	 * @param y
	 *            destination y coordinate
	 */
	public void doMoveTo(int nx, int ny) {
		doMoveTo(nx, ny, true);
	}

	/**
	 * teleport this unit to its destination
	 * 
	 */
	public void teleport() {
		for (int i = 0; i < number; i++) {
			((Warrior) models.get(i)).teleport();
		}
		for (int i = 0; i < signs.size(); i++) {
			((Sign) signs.get(i)).teleport();
		}
	}

	/**
	 * do rotate this unit
	 * 
	 * @param theta
	 *            rotation angle
	 */
	public void doRotate(double theta) {
		orientation += theta;
		orientation = orientation % (2 * java.lang.Math.PI);
		for (int i = 0; i < number; i++) {
			((Warrior) models.get(i)).setAngle(orientation);
		}
		calculateWarriorsPositions();
	}

	/**
	 * reorganize the unit
	 * 
	 */
	public void reorganize() {
		if (isDisorganized()) {
			if (FB.isNet()) {
				Client.getInstance().sendAndWait(MessagesConstants.REORGANIZE + getId());
			} else {
				doReorganize();
			}
		}
	}

	/**
	 * do reorganize the unit
	 * 
	 */
	public void doReorganize() {
		setIsDisorganized(false);
		consumeMovement(4);
		organizationsign.clear();
		String message = Resources.getString(genre + "reorganized");
		message = StringTools.replaceAll(message, "<entity full name>",
				fullName);
		FB.getInstance().message(message, getColor());
	}

	/**
	 * left 90 manoeuvre
	 * 
	 */
	public void left90() {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.LEFT90 + getId());
		} else {
			doLeft90();
		}
	}

	/**
	 * right 90 manoeuvre
	 * 
	 */
	public void right90() {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.RIGHT90 + getId());
		} else {
			doRight90();
		}
	}

	/**
	 * get number of warriors on the last row
	 * 
	 * @return number of warriors on the last row
	 */
	private int getNbLastRow() {
		return front - ((front * nbRows) - number);
	}

	/**
	 * do left 90 manoeuvre
	 * 
	 */
	public void doLeft90() {
		SoundPlayer.getInstance().play("left");
		List newModels = new ArrayList();
		int newX, newY, newFront = nbRows, newNbRows, row, column, oldIndex, nni = 0, nbLastRow = getNbLastRow();

		newNbRows = number / newFront;
		if (number % newFront != 0) {
			newNbRows++;
		}
		for (int i = 0; i < number; i++) {
			column = 1 + i / newFront;
			row = newFront - (i % newFront);
			if (column > nbLastRow) {
				row--;
				if (row == 0) {
					column = front - nni / (newFront - 1);
					row = 1 + (nni % (newFront - 1));
					nni++;
				}
			}
			oldIndex = (column - 1) + (row - 1) * front;
			newModels.add(models.get(oldIndex));
		}
		newX = (int) ((-(float) (front - 1) / 2) * getWidth() * 8 + x);
		if (front % 2 == 0) {
			newX -= (int) (getWidth() * 8 / 2);
		}
		newY = (int) ((nbRows - 1) * getLength() * 8 / 2 + y);
		if (nbRows % 2 == 0) {
			newY -= (int) (getLength() * 8 / 2);
		}
		Point p = new Point(newX, newY);
		GeometryTools.rotatePoint(orientation, p, x, y);
		x = (int) p.getX();
		y = (int) p.getY();
		models = newModels;
		setFront(nbRows);
		doRotate(Math.PI / 2);
	}

	/**
	 * do right 90 manoeuvre
	 * 
	 */
	public void doRight90() {
		SoundPlayer.getInstance().play("right");
		List newmodels = new ArrayList();
		int newX, newY, newFront = nbRows, newNbRows, row, column, oldIndex, nni = 0, nblastrow = getNbLastRow();

		newNbRows = number / newFront;
		if (number % newFront != 0) {
			newNbRows++;
		}
		for (int i = 0; i < number; i++) {
			column = front - i / newFront;
			row = 1 + (i % newFront);
			if (row == nbRows) {
				column -= (front - nblastrow);
				if (column <= 0) {
					column = 1 + nni / (newFront - 1);
					row = (newFront - 1) - (nni % (newFront - 1));
					nni++;
				}
			}
			oldIndex = (column - 1) + (row - 1) * front;
			newmodels.add(models.get(oldIndex));
		}
		newX = (int) (((float) (front - 1) / 2) * getWidth() * 8 + x);
		if (front % 2 == 0) {
			newX -= (int) (getWidth() * 8 / 2);
		}
		newY = (int) ((nbRows - 1) * getLength() * 8 / 2 + y);
		if (nbRows % 2 == 0) {
			newY += (int) (getLength() * 8 / 2);
		}
		Point p = new Point(newX, newY);
		GeometryTools.rotatePoint(orientation, p, x, y);
		x = (int) p.getX();
		y = (int) p.getY();
		models = newmodels;
		setFront(nbRows);
		doRotate(-Math.PI / 2);
	}

	/**
	 * left wheel manoeuvre
	 * 
	 * @param theta
	 *            wheel angle
	 */
	public void leftWheel(double theta) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.LEFTWHEEL + getId() + "#" + theta);
		} else {
			doLeftWheel(theta);
		}
	}

	/**
	 * do left wheel manoeuvre
	 * 
	 * @param theta
	 *            wheel angle
	 */
	public void doLeftWheel(double theta) {
		SoundPlayer.getInstance().play("left");
		consumeMovement(2);
		Warrior model = (Warrior) models.get(0);
		Point p = new Point(x, y);
		GeometryTools.rotatePoint(theta, p, model.getX(), model.getY());
		x = (int) p.getX();
		y = (int) p.getY();
		doRotate(theta);
	}

	/**
	 * right wheel manoeuvre
	 * 
	 * @param theta
	 *            wheel angle
	 */

	public void rightWheel(double theta) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.RIGHTWHEEL + getId() + "#" + theta);
		} else {
			doRightWheel(theta);
		}
	}

	/**
	 * do right wheel manoeuvre
	 * 
	 * @param theta
	 *            wheel angle
	 */
	public void doRightWheel(double theta) {
		SoundPlayer.getInstance().play("right");
		consumeMovement(2);
		Warrior model = (Warrior) models.get(front - 1);
		Point p = new Point(x, y);
		GeometryTools.rotatePoint(theta, p, model.getX(), model.getY());
		x = (int) p.getX();
		y = (int) p.getY();
		doRotate(theta);
	}

	/**
	 * left wheel manoeuvre
	 */
	public void leftWheel() {
		leftWheel(java.lang.Math.PI / 4);
	}

	/**
	 * right wheel manoeuvre
	 */
	public void rightWheel() {
		rightWheel(java.lang.Math.PI / -4);
	}

	/**
	 * expend front line
	 * 
	 * @param plus
	 *            number of added wariors
	 */
	public void expend(int plus) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.EXPEND + getId() + "#" + plus);
		} else {
			doExpend(plus);
		}
	}

	/**
	 * do expend front line
	 * 
	 * @param plus
	 *            number of added wariors
	 */
	public void doExpend(int plus) {
		consumeMovement(2);
		SoundPlayer.getInstance().play("position");
		Object model;
		int i = 0;
		try {

			for (i = 0; i < plus; i++) {
				if (front % 2 == 0) {
					model = models.get(number - 1);
					models.add(front, model);
					models.remove(number);
				} else {
					model = models.get(front * (nbRows - 1));
					models.add(0, model);
					models.remove(front * (nbRows - 1) + 1);
				}
				setFront(front + 1);
				calculateWarriorsPositions();
			}
		} catch (Exception e) {
			FB.exception(e);
		}
	}

	/**
	 * change number of warriors on front line
	 * 
	 * @param front
	 *            new number of warriors on front line
	 */
	public void setFront(int front) {
		if (front > number) {
			FB
					.exception(new Exception(
							"FRONT>NUMBER ---------------------------------------------- "));
		}
		if (front <= 0) {
			FB
					.exception(new Exception(
							"FRONT<=0 ---------------------------------------------- "));
		}
		this.front = front;
	}

	/**
	 * reduce front line
	 * 
	 * @param minus
	 *            number of removed wariors
	 */
	public void reduce(int minus) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.REDUCE + getId() + "#" + minus);
		} else {
			doReduce(minus);
		}
	}

	/**
	 * do reduce front line
	 * 
	 * @param minus
	 *            number of removed wariors
	 */
	public void doReduce(int minus) {
		consumeMovement(2);
		SoundPlayer.getInstance().play("position");
		Object model;
		try {
			for (int i = 0; i < minus; i++) {
				if (front % 2 == 0) {
					model = models.get(0);
					models.add(number - (number % (front - 1)), model);
					models.remove(0);
				} else {
					model = models.get(front - 1);
					models.add(number - 1, model);
					models.remove(front - 1);
				}
				setFront(front - 1);
				calculateWarriorsPositions();
			}
		} catch (Exception e) {
			FB.exception(e);
		}
	}

	// ------------------------------------manoeuvre tests
	// -------------------------------------

	/**
	 * can this unit do a left 90 manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canLeft90() {
		Area obstacles, parcours;
		parcours = left90Path();
		obstacles = getMovementObstacles();
		obstacles.subtract(this.area);
		parcours.intersect(obstacles);
		return parcours.isEmpty();
	}

	/**
	 * can this unit do a right 90 manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canRight90() {
		Area obstacles, parcours;
		parcours = right90Path();
		obstacles = getMovementObstacles();
		obstacles.subtract(this.area);
		parcours.intersect(obstacles);
		return parcours.isEmpty();
	}

	/**
	 * can this unit do a reorganization manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canReorganize() {
		if (!isDisorganized() || getRemainingMove() < 4 || !isMoveAble()
				|| getState() == 'b') {
			return false;
		}
		return true;
	}

	/**
	 * can this unit do a left wheel manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canleftwheel() {
		if (getRemainingMove() < 2 || !isMoveAble()) {
			return false;
		}
		Area obstacles, parcours;
		parcours = wheelPath(java.lang.Math.PI / 4, (Warrior) models.get(0));
		obstacles = getMovementObstacles();
		obstacles.subtract(this.area);
		parcours.intersect(obstacles);
		return parcours.isEmpty();
	}

	/**
	 * can this unit do a right wheel manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canRightWheel() {
		if (getRemainingMove() < 2 || !isMoveAble()) {
			return false;
		}
		Area obstacles, parcours;
		parcours = wheelPath(-java.lang.Math.PI / 4, (Warrior) models.get(0));
		obstacles = getMovementObstacles();
		obstacles.subtract(this.area);
		parcours.intersect(obstacles);
		return parcours.isEmpty();
	}

	/**
	 * calculate left 90 manoeuvre path
	 * 
	 * @return left 90 manoeuvre path area
	 */
	public Area left90Path() {
		return UnitTools.left90PathArea(getFront(), getNumber(),
				getOrientation(), getX(), getY(), getWidth(), rowLength(),
				getNbRows(), getVertex());
	}

	/**
	 * calculate right 90 manoeuvre path
	 * 
	 * @return right 90 manoeuvre path area
	 */
	public Area right90Path() {
		return UnitTools.right90PathArea(getFront(), getNumber(),
				getOrientation(), getX(), getY(), getWidth(), rowLength(),
				getNbRows(), getVertex());
	}

	/**
	 * calculate wheel manoeuvre path
	 * 
	 * @return wheel manoeuvre path area
	 */
	public Area wheelPath(double theta, Warrior model) {
		return UnitTools.wheelPathArea(vertex, getX(), getY(), theta, model);
	}

	/**
	 * can this unit do a surround manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canSurround() {
		if (!isMoveAble() && !surrounding && getRemainingMove() >= 1) {
			for (int i = 0; i < front; i++) {
				if (((Warrior) models.get(i)).getEnnemyContacts().size() == 0) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * can this unit do an expend manoeuvre
	 * 
	 * @return number of warriors that can be added to front line
	 */
	public int canExpend() {
		if (front == number || getRemainingMove() < 2) {
			return 0;
		}
		Area obstacles, testarea;
		int i;
		for (i = 1; i < 8 && i < number && front + i < number; i++) {
			testarea = expendArea(i);
			obstacles = getMovementObstacles();
			obstacles.subtract(this.area);
			testarea.intersect(obstacles);
			if (!testarea.isEmpty()) {
				return i - 1;
			}
		}
		return i;
	}

	/**
	 * can this unit do a reduce manoeuvre
	 * 
	 * @return number of warriors that can be removed from front line
	 */
	public int canReduce() {
		if (front == 1 || getRemainingMove() < 2) {
			return 0;
		}
		Area obstacles, testarea;
		int i;
		for (i = 1; i < 8 && i < front; i++) {
			testarea = reduceArea(i);
			obstacles = getMovementObstacles();
			obstacles.subtract(this.area);
			testarea.intersect(obstacles);
			if (!testarea.isEmpty()) {
				return i - 1;
			}
		}
		return i;
	}

	/**
	 * 
	 * calculate expended area
	 * 
	 * @param plus
	 *            number of warriors added to front line
	 * @return expended area
	 */
	public Area expendArea(int plus) {
		Point[] result = UnitTools.calculateVertex(getFront() + plus,
				getNumber(), getOrientation(), getX(), getY(), getWidth(),
				rowLength());
		return UnitTools.vertexToArea(result);
	}

	/**
	 * 
	 * calculate reduced area
	 * 
	 * @param minus
	 *            number of warriors removed from front line
	 * @return reduced area
	 */
	public Area reduceArea(int minus) {
		Point[] result = UnitTools.calculateVertex(getFront() - minus,
				getNumber(), getOrientation(), getX(), getY(), getWidth(),
				rowLength());
		return UnitTools.vertexToArea(result);
	}

	/**
	 * get row length
	 * 
	 * @return row length
	 */
	public float rowLength() {
		return getLength();
	}

	/**
	 * get row width
	 * 
	 * @return row width
	 */
	public float rowWidth(int row) {
		if (row == nbRows - 1) {// last row
			return getNbLastRow() * getWidth();
		}
		return getFront() * getWidth();
	}

	// -------------------------------------------------anims-------------------------------------------
	/**
	 * is this unit walking
	 * 
	 * @return true if one of the warriors of this unit is walking
	 */
	public boolean isWalking() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			if (model.iswalking()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * is this unit dying
	 * 
	 * @return true if one of the warriors of this unit is dying
	 */
	public boolean isDying() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			if (model.getAction() == 'd') {
				return true;
			}
		}
		return false;
	}

	/**
	 * stop unit action and make it stand
	 * 
	 */
	public void stand() {
		if (isActive()) {
			Warrior model;
			for (int i = 0; i < number; i++) {
				model = getModel(i);
				if (model.getAction() != 'd' && model.getAction() != 'x') {
					model.setAction('n');
					model.attribseq();
				}
			}
		}
	}

	/**
	 * disorganize entity animations
	 * 
	 */
	public void disorganizeAnim() {
		for (int i = 0; i < number; i++) {
			((Warrior) models.get(i)).setAnimationCounter(DX.roll(4));
		}
	}

	/**
	 * organize entity animations
	 * 
	 */
	public void organizeAnim() {
		for (int i = 0; i < number; i++) {
			((Warrior) models.get(i)).setAnimationCounter(1);
		}
	}

	/**
	 * start fight animation
	 * 
	 */
	public void animFight() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			if ((model.getEnnemyContacts().size() != 0)
					|| (model.getEnnemySpearContacts().size() != 0 && twoRows)
					|| (model.getEnnemyPikeContacts().size() != 0 && twoRows && weapon == 'p')) {
				model.animfight();
			}
		}
	}

	/**
	 * start threat animation
	 * 
	 */
	public void animThreat() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			model.setAction('t');
			model.attribseq();
		}
	}

	/**
	 * start threat animation
	 * 
	 * @param target
	 *            threat target
	 */
	public void animThreat(Unit target) {
		double angledethreat = angleTo(target);
		for (int i = 0; i < number; i++) {
			((Warrior) models.get(i)).animthreat(angledethreat);
		}
	}

	/**
	 * start deaths animations
	 * 
	 */
	public void animKill() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			if (model.mustDie()) {
				model.setAction('d');
				model.setAnimationCounter(-DX.roll(10));
			}
		}
	}

	/**
	 * start fast deaths animations
	 * 
	 */
	public void animFastKill() {
		Warrior model;
		for (int i = 0; i < number; i++) {
			model = (Warrior) models.get(i);
			if (model.mustDie()) {
				model.setAction('d');
				model.setAnimationCounter(2 - DX.roll(3));
			}
		}
	}

	// --------------------------------------------------------------------------------------------

	/**
	 * organize warriors positions
	 */

	public void organize() {
		nbRows = 0;
		int reste = number;
		int xd, yd, line;
		for (int i = 0; i < number;) {
			if (reste > front) {
				line = front;
			} else {
				line = reste;
			}
			for (int j = 0; j < line; j++) {
				xd = (int) ((((float) j - (float) (line - 1) / 2) * getWidth() * 8) + x);
				if (front % 2 == 0) {
					xd -= (getWidth() * 8 / 2);
				}
				yd = (int) ((nbRows * getLength() * 8) + y);
				((Warrior) models.get(i)).moveTo(xd, yd);
				i++;
			}
			nbRows++;
			reste -= front;
		}
		organizeSigns();
	}

	/**
	 * start movement animation
	 * 
	 */
	public void animMovement() {
		for (int i = 0; i < number; i++) {
			getModel(i).animMovement();
		}

	}

	/**
	 * organize signs positions
	 */
	public void organizeSigns() {
		int xd, yd;
		for (int i = 0; i < signs.size(); i++) {
			xd = (int) (x + Math.pow(-1, i) * ((i + 1) / 2) * 10);
			yd = (int) ((nbRows * rowLength() * 8) + y + (8 / 2));
			((Sign) signs.get(i)).moveto(xd, yd);
		}
	}

	/**
	 * get unit total worth
	 * 
	 * @return unit total worth
	 */
	public int totalWorth() {
		return number * getWorth();
	}

	/**
	 * test entity level
	 * 
	 * @return test score
	 */
	public int levelTest() {
		return D100.roll() + (level - 1) * 15;
	}

	/**
	 * test target threat
	 * 
	 * @param target
	 *            target unit
	 * @return threat score
	 */
	public int threatScore(Unit target) {
		int score = (target.level - 1) * 15;
		if (target.getState() == 's') {
			score -= 30;
		}
		if (target.closeopposingtc) {
			score -= 30;
		}
		score -= musicians;
		if (target.totalWorth() < totalWorth()) {
			score -= 15;
		}
		if (target.suffersBadLight()) {
			score -= 15;
		}
		if (target.heroWithUnit()) {
			score += 15;
		}
		if (target.musicians > 0) {
			score += 15;
		}
		if (target.hasStandardBearer) {
			score += 15;
		}
		if (target.hasChief) {
			score += 15;
		}
		if (target.getState() == 'b') {
			score += 15;
		}
		return score;
	}

	/**
	 * threat ennemy unit
	 * 
	 * @param target
	 *            ennemy target
	 */
	public void threat(Unit target) {
		if (target.getState() != 'r') {
			int score = D100.roll() + threatScore(target);
			target.reaction(score);
		}
	}

	/**
	 * apply magic threat spell
	 * 
	 */
	public void magicThreat() {
		setMorale(morale - 3);
	}

	/**
	 * calculate morale after casualties
	 * 
	 */
	public void casualtyTest() {
		int score = levelTest();
		if (casualtyPercentage > 50) {
			score -= 45;
		}
		if (getState() == 's') {
			score -= 30;
		}
		if (casualtyThisTurn) {
			score -= 15;
		}
		if (suffersBadLight()) {
			score -= 15;
		}
		if (closeOpposingTC()) {
			score -= 15;
		}
		if (heroWithUnit()) {
			score += 15;
		}
		if (hasStandardBearer) {
			score += 15;
		}
		if (hasChief) {
			score += 15;
		}
		if (getState() == 'b') {
			score += 30;
		}
		reaction(score);
	}

	/**
	 * calculate morale after combat
	 * 
	 */
	public void combatTest() {
		int score = levelTest();
		if (casualtyPercentage > 50) {
			score -= 45;
		}
		if (casualtyPercentage > 25) {
			score -= 15;
		}
		if (getState() == 's') {
			score -= 30;
		}
		if (order == 'o') {
			score -= 15;
		}
		if (currentCasualties == 0) {
			score += 15;
		}

		if (currentCasualties > opponentsCasualties) {
			score -= 15;
		} else if (currentCasualties < opponentsCasualties) {
			score += 15;
		}
		if (hasLostStandard) {
			score -= 15;
		}
		if (suffersBadLight()) {
			score -= 15;
		}
		if (terribleopponent) {
			score -= 15;
		}
		if (hasStandardBearer) {
			score += 15;
		}
		if (order == 'h') {
			score -= 15;
		}
		if (hasChief) {
			score += 15;
		}
		if (heroWithUnit()) {
			score += 15;
		}
		if (getState() == 'b') {
			score += 30;
		}
		reaction(score);
		opponentsCasualties = 0;
		terribleopponent = false;
	}

	/**
	 * calculate morale reaction after casualties or combat
	 */
	public void reaction(int score) {
		int newMorale = 0;
		if (type == 'c') {
			if (score >= 125) {
				newMorale = 10;
			} else if (score >= 50) {
				newMorale = 4 + DX.roll(5);
			} else if (score >= 0) {
				newMorale = DX.roll(4);
			} else {
				newMorale = 0;
			}
		} else if (type == 'a') {
			if (score >= 95) {
				newMorale = 10;
			} else if (score >= 50) {
				newMorale = 4 + DX.roll(5);
			} else if (score >= 20) {
				newMorale = DX.roll(4);
			} else {
				newMorale = 0;
			}
		} else if (type == 'f') {
			if (score >= 80) {
				newMorale = 10;
			} else if (score >= 50) {
				newMorale = 4 + DX.roll(5);
			} else if (score >= 20) {
				newMorale = DX.roll(4);
			} else {
				newMorale = 0;
			}
		}
		setMorale(newMorale);
	}

	/**
	 * change morale value
	 * 
	 * @param morale
	 *            new morale
	 */
	public void setMorale(int morale) {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(
					MessagesConstants.SETMORALE + getId() + "#" + morale);
		} else {
			doSetMorale(morale);
		}
	}

	/**
	 * do change morale value
	 * 
	 * @param morale
	 *            new morale
	 */
	public void doSetMorale(int morale) {
		int oldValue = this.morale;
		this.morale = morale;
		firePropertyChange(MORALE, oldValue, morale);
		if (morale == 10) {
			setBloodlust();
		} else if (morale >= 5) {
			setNormal();
		} else if (morale >= 1) {
			setShaken();
		} else {
			rout();
		}
	}

	// ---------------------------------------------------------------KILLS------------------------------------------------

	/**
	 * get next casualty
	 */
	public Warrior nextTarget() {
		Warrior nexttarget;
		if (nextIndex < deads.size()) {
			nexttarget = (Warrior) deads.get(nextIndex);
			nextIndex++;
		} else {
			nexttarget = (Warrior) models.get(DX.roll(models.size()) - 1);
		}
		return nexttarget;
	}

	/**
	 * add a warrior to deads list
	 */
	public void setDead(Warrior model) {
		deads.add(model);
		currentCasualties++;
		nextIndex = 0;
	}

	/**
	 * wound this unit
	 * 
	 * @param wounds
	 *            number of wounds
	 * @return remaining wounds
	 */
	public int wound(int wounds) {
		int casualties = 0;
		Warrior model;
		List woundeds = setWoundeds();
		for (int i = 0; i < wounds && woundeds.size() != 0; i++) {
			int index = DX.roll(woundeds.size()) - 1;
			model = (Warrior) woundeds.get(index);
			model.wound(1);
			if (model.getLifePoints() <= 0) {
				casualties++;
				woundeds.remove(model);
			}
			if (woundeds.size() == 0) {
				woundeds = setWoundeds();
			}
		}
		return casualties;
	}

	/**
	 * calculate woundeds
	 * 
	 * @return woundeds list
	 */
	public List setWoundeds() {
		List woundeds = new ArrayList();
		Warrior model;
		if (Battle.getLoop().getCurrentPhase() instanceof CombatPhase) {
			for (int i = 0; i < number; i++) {
				model = (Warrior) models.get(i);
				if (model.getLifePoints() > 0
						&& model.getEnnemyContacts().size() != 0) {
					woundeds.add(model);
				}
			}
			if (woundeds.size() == 0) {
				for (int i = 0; i < number; i++) {
					model = (Warrior) models.get(i);
					if (model.getLifePoints() > 0
							&& model.getEnnemySpearContacts().size() != 0) {
						woundeds.add(model);
					}
				}
			}
			if (woundeds.size() == 0) {
				for (int i = 0; i < number; i++) {
					model = (Warrior) models.get(i);
					if (model.getLifePoints() > 0
							&& model.getEnnemyPikeContacts().size() != 0) {
						woundeds.add(model);
					}
				}
			}
			if (woundeds.size() == 0) {
				for (int i = 0; i < number; i++) {
					model = (Warrior) models.get(i);
					if (model.getLifePoints() > 0) {
						woundeds.add(model);
					}
				}
			}
		} else {
			for (int i = 0; i < number; i++) {
				model = (Warrior) models.get(i);
				if (model.getLifePoints() > 0) {
					woundeds.add(model);
				}
			}
		}
		return woundeds;
	}

	/**
	 * remove dead warriors after killing
	 * 
	 */
	public void commitKill() {
		if (getState() != 'a') {
			String message;
			Warrior model;
			int oldnumber = number;
			int tmpnumber = number - currentCasualties;
			if (tmpnumber < 0) {
				tmpnumber = 0;
			}
			setNumber(tmpnumber);

			casualtyPercentage = (oldnumber - number) * 100 / oldnumber;
			if (currentCasualties != 0) {
				// enlever les figurines
				int indexremplacant = models.size() - 1;
				Warrior remplacant = null;
				boolean yadr = true, found = false;
				for (int i = 0; i <= indexremplacant; i++) {
					model = (Warrior) models.get(i);
					if (model.getLifePoints() <= 0) {
						// enlever le cadavre
						model.clearcontacts();
						BattleFieldModel.getInstance().removeWarrior(model);
						deads.remove(model);
						// chercher un remplacant
						found = false;
						while (!found && yadr) {
							remplacant = (Warrior) models
									.remove(indexremplacant);
							indexremplacant--;
							if (indexremplacant <= i) {
								yadr = false;
							}
							if (remplacant.getLifePoints() > 0) {
								found = true;
							} else {
								remplacant.clearcontacts();
								BattleFieldModel.getInstance().removeWarrior(
										remplacant);
								deads.remove(remplacant);
							}
						}
						// remplacer si on a trouve
						if (found) {
							models.set(i, remplacant);
						}
					}
				}

				if (currentCasualties != 1) {
					message = Resources.getString(genre + "casualties");
					message = StringTools.replaceAll(message, "<casualties>",
							Integer.toString(currentCasualties));
				} else {
					message = Resources.getString(genre + "1casualty");
				}
			} else {
				message = Resources.getString(genre + "nocasualty");
			}
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			FB.getInstance().message(message, getColor());
			if (number == 0) {
				setAnnihilated();
			} else {
				calculateWarriorsPositions();
			}
			currentCasualties = 0;
		}
	}

	/**
	 * combat another entity
	 * 
	 * @param combatworth
	 *            number of dice
	 * @param target
	 *            ennemy target
	 * @return number of wounds
	 */
	public int combat(int combatworth, Entity target) {
		// --hits
		int dicetohit = combatworth;
		if (hasChampion) {
			dicetohit++;
		}
		if (order == 'a') {
			int fiveuplet = (int) dicetohit / 5;
			dicetohit += fiveuplet;
		}
		int hits = 0, tohit = 50;
		if (level < target.level) {
			tohit += 15;
		} else if (level > target.level) {
			tohit -= 15;
		}
		if (twoRows) {
			tohit += 15;
		}
		if (isDisorganized()) {
			tohit += 15;
		}
		if (!target.hasShield) {
			tohit -= 15;
		}
		for (int i = 0; i < dicetohit; i++) {
			int d100 = D100.roll();
			if (d100 >= D100.LUCKY) {
				hits++;
			} else if (d100 < D100.FUMBLE) {
			} else if (d100 >= tohit) {
				hits++;
			}
		}
		// --kills
		int kills = 0, tokill;
		tokill = rollToKill(target, weapon);
		tokill -= 15;
		tokill -= (getStrength() * 15);
		tokill += (target.getResistance() * 15);
		if (getState() == 'b') {
			tokill -= 15;
		}
		for (int i = 0; i < hits; i++) {
			int d100 = D100.roll();
			if (d100 >= D100.LUCKY) {
				kills++;
			} else if (d100 < D100.FUMBLE) {
			} else if (d100 >= tokill) {
				kills++;
			}
		}
		target.wound(kills);
		return kills;
	}

	/**
	 * bloodlust reaction
	 * 
	 */
	public void setBloodlust() {
		if (getState() != 'b') {
			setState('b');
			statesign.setImage(Resources.getSign("bloodlust"));
			String message = Resources.getString(genre + "bloodlust");
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			SoundPlayer.getInstance().play("bloodlust");
			FB.getInstance().message(message, getColor());
		}
	}

	/**
	 * normal reaction
	 * 
	 */
	public void setNormal() {
		if (getState() != 'n') {
			setState('n');
			statesign.clear();
			String message = Resources.getString(genre + "normal");
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			FB.getInstance().message(message, getColor());
		}
	}

	/**
	 * shaken reaction
	 * 
	 */
	public void setShaken() {
		if (getState() != 's') {
			setState('s');
			statesign.setImage(Resources.getSign("shaken"));
			String message = Resources.getString(genre + "shaken");
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			SoundPlayer.getInstance().play("shaken");
			FB.getInstance().message(message, getColor());
		}
		disorganize();
	}

	/**
	 * disorganize unit
	 * 
	 */
	public void disorganize() {
		if (!isDisorganized()) {
			setIsDisorganized(true);
			organizationsign.setImage(Resources.getSign("disorganized"));
			String message = Resources.getString(genre + "disorganized");
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			if (FB.getInstance() != null) {

				FB.getInstance().message(message, getColor());
			}
		}
	}

	/**
	 * unit routs
	 */
	public void rout() {
		if (getState() != 'r' && getState() != 'a') {
			setState('r');
			// derouter les figurines
			for (int i = 0; i < number; i++) {
				((Warrior) models.get(i)).rout();
			}
			for (int i = 0; i < signs.size(); i++) {
				BattleFieldModel.getInstance().removeSign((Sign) signs.get(i));
			}
			signs.clear();
			String message = Resources.getString(genre + "rout");
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			SoundPlayer.getInstance().play("rout");
			FB.getInstance().message(message, getColor());
		}
	}

	/**
	 * remove this unit after routing
	 * 
	 */
	public void commitRout() {
		command.removeActiveEntity(this);
		clearContacts();
		Battle.getInstance().getActiveEntities().remove(this);
		// enlever les figurines
		Warrior figure;
		lifePointsHolder = new int[number];
		for (int i = 0; i < number; i++) {
			figure = (Warrior) models.get(i);
			BattleFieldModel.getInstance().removeWarrior(figure);
			lifePointsHolder[i] = figure.getLifePoints();
		}

	}

	/**
	 * remove this unit after annihilation
	 * 
	 */
	public void setAnnihilated() {
		if (getState() != 'a') {
			setState('a');
			command.removeActiveEntity(this);
			String message = Resources.getString(genre + "annihilated");
			message = StringTools.replaceAll(message, "<entity full name>",
					fullName);
			FB.getInstance().message(message, getColor());
			clearContacts();
			Battle.getInstance().getActiveEntities().remove(this);
			for (int i = 0; i < signs.size(); i++) {
				BattleFieldModel.getInstance().removeSign((Sign) signs.get(i));
			}
		}
	}

	/**
	 * is there a hero close to this unit
	 * 
	 * @return true if there is a friendly hero close to this unit
	 */
	public boolean heroWithUnit() {
		Entity entity;
		for (int i = 0; i < friendlycontacts.size(); i++) {
			entity = (Entity) (friendlycontacts.get(i));
			if (entity.isHero()) {
				return true;
			}
		}
		return false;
	}

	/**
	 * is there a frightning warrior close to this unit
	 * 
	 * @return true if there is a frightning ennemy warrior close to this unit
	 */
	public boolean closeOpposingTC() {
		Entity entity;
		for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) (Battle.getInstance().getActiveEntities().get(i));
			if (entity.terrible && lineOfSightTo(entity) < 6 * 8
					&& isEnnemy(entity)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * can this unit move
	 * 
	 * @return true if this unit can move
	 */
	public boolean isMoveAble() {
		return super.isMoveAble() && order != 'h';
	}

	/**
	 * surround manoeuvre
	 * 
	 */
	public void surround() {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.SURROUND + getId());
		} else {
			doSurround();
		}
	}

	/**
	 * get warrior index
	 * 
	 * @param row
	 *            warrior row
	 * @param column
	 *            warrior column
	 * @return warrior index
	 */
	private int getModelIndex(int row, int column) {
		return column + row * front;
	}

	/**
	 * do surround manoeuvre
	 * 
	 */
	public void doSurround() {
		calculateSurroundPositions();
		animMovement();
	}

	/**
	 * calculate warrios positions for surrounding
	 * 
	 */
	public void calculateSurroundPositions() {

		consumeMovement(1);
		if (FB.getInstance() != null) {
			SoundPlayer.getInstance().play("attack");
		}
		surrounding = true;
		try {
			if (opposingContacts.size() != 0) {
				List surrounders, ennemyModels, possibilities, founds;

				int surrounderColumn;
				int surrounderRow;
				int surrounderRowLength;
				int surrounderIndex;

				int nblastrow = getNbLastRow();
				Warrior surrounder, ennemyModel;
				Warrior.SurroundingPossibility possibility;
				Area normalarea;
				boolean stop;
				Warrior.SurroundingPossibility closest;
				Point origin;

				Area unitSecondArea = new Area();
				for (int i = 0; i < number; i++) {
					unitSecondArea.add(((Warrior) models.get(i))
							.getSpearContactArea());
				}
				unitSecondArea.subtract(area);

				surrounders = new ArrayList();
				for (int i = 0; i < number; i++) {
					surrounderRow = i / front;
					surrounderRowLength = (surrounderRow == (nbRows - 1)) ? nblastrow
							: front;
					if (i % 2 == 0) {
						surrounderColumn = (i - front * surrounderRow) / 2;
					} else {
						surrounderColumn = surrounderRowLength - 1
								- (i - front * surrounderRow) / 2;
					}

					surrounderIndex = getModelIndex(surrounderRow,
							surrounderColumn);

					if (((Warrior) models.get(surrounderIndex))
							.getEnnemyContacts().size() == 0) {
						surrounders.add(models.get(surrounderIndex));
					}
				}

				if (surrounders.size() != 0) {

					Entity previousEnnemy = null, ennemy;

					Area obstacles = getMovementObstacles();
					obstacles.add(area);
					possibilities = new ArrayList();
					for (int i = 0; i < opposingContacts.size(); i++) {

						ennemy = (Entity) opposingContacts.get(i);

						if (ennemy != previousEnnemy) {
							ennemyModels = ennemy.getModels();
							for (int j = 0; j < ennemyModels.size(); j++) {
								ennemyModel = (Warrior) ennemyModels.get(j);
								possibilities
										.addAll(ennemyModel
												.calculateSurroundinPossibilities(
														getWidth() * 8,
														getLength() * 8));
							}
						}
						previousEnnemy = ennemy;

					}

					stop = false;
					for (int k = 0; k < surrounders.size() && !stop; k++) {
						surrounder = (Warrior) surrounders.get(k);

						founds = new ArrayList();
						// lever la figurine
						area.subtract(surrounder.getBaseArea());
						unitSecondArea.subtract(surrounder
								.getSpearContactArea());

						for (int l = possibilities.size() - 1; l >= 0; l--) {
							possibility = (Warrior.SurroundingPossibility) possibilities
									.get(l);

							if (obstacles.contains(possibility.position)) {
								possibilities.remove(possibility);
							} else {

								/*
								 * if (unitSecondArea
								 * .contains(possibility.position)) {
								 */
								if (testSurroundingPossibility(surrounder,
										possibility, obstacles)) {
									founds.add(possibility);

								}
								// }

							}
						} // for possibilities
						if (founds.isEmpty()) {
							stop = true;

						} else {

							closest = (Warrior.SurroundingPossibility) founds
									.get(0);
							origin = new Point(surrounder.getXd(), surrounder
									.getYd());
							for (int n = 1; n < founds.size(); n++) {

								if (origin
										.distance(((Warrior.SurroundingPossibility) founds
												.get(n)).position) <= origin
										.distance(closest.position)) {
									closest = (Warrior.SurroundingPossibility) founds
											.get(n);
								}
							}

							surrounder.moveTo(closest.position.x,
									closest.position.y);

							surrounder.setAngle(closest.theta);
						}
						// reposer la figurine
						area.add(surrounder.getBaseArea());
						obstacles.add(surrounder.getBaseArea());

						unitSecondArea.add(surrounder.getSpearContactArea());
					} // for surrounders
					calculateContacts();
					normalarea = UnitTools.vertexToArea(vertex);
					// anti bug hole in area when wheel and surround
					area.add(normalarea);

				}

			}
		} catch (Exception e) {
			FB.exception(e);
		}

	}

	/**
	 * test if a surrounding possibility can be used by a warrior
	 * 
	 * @param model
	 *            the warrior
	 * @param possibility
	 *            the surrounding possibility
	 * @param obstacles
	 *            the unit obstacles
	 * @return true if the surrounding possibility can be used by the warrior
	 */
	public boolean testSurroundingPossibility(Warrior model,
			Warrior.SurroundingPossibility possibility, Area obstacles) {

		double deltaTheta = possibility.theta - model.getAngle();

		Area socle = GeometryTools.transformedArea(model.getBaseArea(), model
				.getXd(), model.getYd(), possibility.position.x,
				possibility.position.y, -deltaTheta);

		Area contactarea = new Area(model.getContactArea());
		contactarea.add(model.getSpearContactArea());
		contactarea = GeometryTools.transformedArea(contactarea, model.getXd(),
				model.getYd(), possibility.position.x, possibility.position.y,
				-deltaTheta);

		Area intersocleobstacle = socle;
		Area intercontactunit = contactarea;

		intersocleobstacle.intersect(obstacles);
		intercontactunit.intersect(area);

		if (intersocleobstacle.isEmpty() && !intercontactunit.isEmpty()) {
			return true;
		}

		return false;
	}

	/**
	 * returns unit as a string to save it to a file
	 * 
	 * @return unit as a string
	 */

	public String saveString() {
		String result = super.saveString() + '\t' + initialNumber + '\t'
				+ number + '\t' + morale + '\t' + front + '\t'
				+ casualtyPercentage + '\t' + musicians + '\t'
				+ opponentsCasualties + '\t' + (upgraded ? 'y' : 'n') + '\t'
				+ (hasChief ? 'y' : 'n') + '\t' + (hasChampion ? 'y' : 'n')
				+ '\t' + (hasStandardBearer ? 'y' : 'n') + '\t'
				+ (surrounding ? 'y' : 'n') + '\t'
				+ (isDisorganized() ? 'y' : 'n') + '\t'
				+ (casualtyThisTurn ? 'y' : 'n') + '\t'
				+ (changedOrders ? 'y' : 'n');
		if (models != null) {
			for (int i = 0; i < number; i++) {
				result += ("\t" + ((Warrior) models.get(i)).getLifePoints());
			}
		} else if (lifePointsHolder != null) {
			for (int i = 0; i < number; i++) {
				result += ("\t" + lifePointsHolder[i]);
			}
		} else {
			for (int i = 0; i < number; i++) {
				result += ("\t" + initialPdv);
			}
		}
		result += "\t" + order;
		return result;
	}

	/**
	 * load unit from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		initialNumber = StringTools.readBufferInt(buffy);
		number = StringTools.readBufferInt(buffy);
		morale = StringTools.readBufferInt(buffy);
		front = StringTools.readBufferInt(buffy);
		casualtyPercentage = StringTools.readBufferFloat(buffy);
		musicians = StringTools.readBufferInt(buffy);
		opponentsCasualties = StringTools.readBufferInt(buffy);
		upgraded = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
		hasChief = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
		hasChampion = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
		hasStandardBearer = (StringTools.readBufferChar(buffy) == 'y') ? true
				: false;
		surrounding = (StringTools.readBufferChar(buffy) == 'y') ? true : false;
		disorganized = (StringTools.readBufferChar(buffy) == 'y') ? true
				: false;
		casualtyThisTurn = (StringTools.readBufferChar(buffy) == 'y') ? true
				: false;
		changedOrders = (StringTools.readBufferChar(buffy) == 'y') ? true
				: false;
		lifePointsHolder = new int[number];
		for (int i = 0; i < number; i++) {
			lifePointsHolder[i] = StringTools.readBufferInt(buffy);
		}
		if (buffy.length() > 0) {
			order = StringTools.readBufferChar(buffy);
		}
	}

	/**
	 * Sets the x.
	 * 
	 * @param x
	 *            The x to set
	 */
	public void setX(int x) {
		this.x = x;
	}

	/**
	 * Sets the y.
	 * 
	 * @param y
	 *            The y to set
	 */
	public void setY(int y) {
		this.y = y;
	}

	/**
	 * get resource key
	 */
	protected String getResourceKey() {
		if (weapon == 's' || weapon == 'p') {
			return "spearunit";
		} else if (weapon == '2') {
			return "2munit";
		} else {
			return "1munit";
		}
	}

	/**
	 * shrink this unit
	 * 
	 */
	public void shrink() {
		super.shrink();
		Iterator it = models.iterator();
		while (it.hasNext()) {
			((Warrior) it.next()).resetAreas();
		}
		calculateWarriorsPositions();
	}

	/**
	 * heal this unit
	 * 
	 * @param lifePoints
	 *            added life points
	 */
	public void heal(int healedLifePoints) {
		Iterator iter = models.iterator();
		Warrior model;
		while (healedLifePoints > 0) {
			if (!iter.hasNext()) {
				iter = models.iterator();
			}
			model = (Warrior) iter.next();
			model.heal(1);
			healedLifePoints--;
		}
	}

	/**
	 * get unit morale
	 * 
	 * @return unit morale
	 */
	public int getMorale() {
		return morale;
	}

	/**
	 * get warriors number as a string
	 * 
	 * @return warriors number as a string
	 */
	public String getNumberString() {
		return Integer.toString(number);
	}

	/**
	 * get number of warriors
	 * 
	 * @return number of warriors
	 */
	public int getNumber() {
		return number;
	}

	/**
	 * change number of warriors
	 * 
	 * @param number
	 *            new number of warriors
	 */
	public void setNumber(int number) {
		int oldValue = this.number;
		this.number = number;
		if (number == 0) {
			setAnnihilated();
		} else if (front > number) {
			setFront(number);
		}
		firePropertyChange(NUMBER, oldValue, number);
	}

	/**
	 * rotate unit to face another entity
	 * 
	 * @param e
	 *            the other entity
	 */
	public void rotateTo(Entity e) {
		int angleto = GeometryTools.to8Base(angleTo(e) - getAngle()) / 2;
		if (angleto <= 2) {
			boolean stop = false;
			for (int i = 0; i < angleto && !stop; i++) {
				if (canLeft90()) {
					left90();
				} else {
					stop = true;
				}
			}
		} else {
			boolean stop = false;
			for (int i = 4; i > angleto && !stop; i--) {
				if (canRight90()) {
					right90();
				} else {
					stop = true;
				}
			}
		}
	}

	/**
	 * number of warriors for this unit
	 * 
	 * @return number of warriors
	 */
	public int numberOfModels() {
		return number;
	}

	/**
	 * @return Returns the disorganized.
	 */
	public synchronized boolean isDisorganized() {
		return disorganized;
	}

	/**
	 * @param disorganized
	 *            The disorganized to set.
	 */
	public synchronized void setIsDisorganized(boolean isdisorganized) {
		this.disorganized = isdisorganized;
		firePropertyChange(DISORGANIZED, !isdisorganized, isdisorganized);
	}

	/**
	 * get maximum number of warriors that can be added for expend manoeuvre
	 * 
	 * @return maximum number of warriors that can be added
	 */

	public int maxExpend() {
		return getNumber() - getFront();
	}

	/**
	 * get initial number of warriors as a string
	 * 
	 * @return initial number of warriors as a string
	 */
	public String getInitialNumberString() {
		return Integer.toString(initialNumber);
	}

	/**
	 * set initial number of wariors
	 * 
	 * @param initialnumber
	 *            new initial number of wariors
	 */
	public void setInitialNumber(int initialnumber) {
		this.initialNumber = initialnumber;
		lifePointsHolder = null;
		setNumber(initialNumber);
	}

	/**
	 * reset unit remaining movement points
	 */
	public void resetRemainingMove() {
		super.resetRemainingMove();
		resetChangedOrders();
	}

	/**
	 * reset unit changedOrders attribute
	 * 
	 */
	public void resetChangedOrders() {
		changedOrders = false;
	}

	/**
	 * get number of warriors
	 * 
	 * @return number of warriors
	 */
	public int getNumberOfWarriors() {
		return getNumber();
	}

	/**
	 * @return Returns the hasChampion.
	 */
	public boolean hasChampion() {
		return hasChampion;
	}

	/**
	 * @param hasChampion The hasChampion to set.
	 */
	public void setHasChampion(boolean hasChampion) {
		this.hasChampion = hasChampion;
	}

	/**
	 * @return Returns the hasStandardBearer.
	 */
	public boolean hasStandardBearer() {
		return hasStandardBearer;
	}

	/**
	 * @param hasStandardBearer The hasStandardBearer to set.
	 */
	public void setHasStandardBearer(boolean hasStandardBearer) {
		this.hasStandardBearer = hasStandardBearer;
	}

	/**
	 * @return Returns the musicians.
	 */
	public int getMusicians() {
		return musicians;
	}

	/**
	 * @param musicians The musicians to set.
	 */
	public void setMusicians(int musicians) {
		this.musicians = musicians;
	}

	/**
	 * @return Returns the surrounding.
	 */
	public boolean isSurrounding() {
		return surrounding;
	}

	/**
	 * @param surrounding The surrounding to set.
	 */
	public void setSurrounding(boolean surrounding) {
		this.surrounding = surrounding;
	}

	/**
	 * @return Returns the upgraded.
	 */
	public boolean isUpgraded() {
		return upgraded;
	}

	/**
	 * @param upgraded The upgraded to set.
	 */
	public void setUpgraded(boolean upgraded) {
		this.upgraded = upgraded;
	}

	/**
	 * @return Returns the casualtyPercentage.
	 */
	public float getCasualtyPercentage() {
		return casualtyPercentage;
	}

	/**
	 * @return Returns the changedOrders.
	 */
	public boolean isChangedOrders() {
		return changedOrders;
	}

	/**
	 * @return Returns the front.
	 */
	public int getFront() {
		return front;
	}

	/**
	 * @return Returns the hasChief.
	 */
	public boolean hasChief() {
		return hasChief;
	}

	/**
	 * @return Returns the initialNumber.
	 */
	public int getInitialNumber() {
		return initialNumber;
	}

	/**
	 * @return Returns the nbRows.
	 */
	public int getNbRows() {
		return nbRows;
	}

	/**
	 * @return Returns the order.
	 */
	public char getOrder() {
		return order;
	}

	/**
	 * @return Returns the orientation.
	 */
	public double getOrientation() {
		return orientation;
	}

	

}