/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.battlefieldelements;

// une magic
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;

import javax.swing.ImageIcon;

import fb.army.entities.SpellCaster;
import fb.battlefield.BattleFieldModel;
import fb.util.GeometryTools;

/**
 * a spell visual effect
 * 
 * @author farfpeuf
 * 
 */
public class MagicEffect extends Animated {
	// origin
	private int x, y;

	// destination
	private int xd, yd;

	// offsets
	private int effectDx, effectDy;

	private int step = 16;

	// pixel size
	private int width, length;

	// associateed icons
	private ImageIcon[] icons = null;

	// current image
	private Image image = null;

	// target warrior
	private Warrior target;

	// will this effect kill its target?
	private boolean killing;

	// the spell caster
	private SpellCaster caster;

	/**
	 * @return Returns the x.
	 */
	public int getX() {
		return x;
	}

	/**
	 * @return Returns the y.
	 */
	public int getY() {
		return y;
	}

	/**
	 * paint the effect on the battlefield
	 */
	public void paintImage(Graphics g, int dx, int dy, int scale) {
		if (image != null) {
			g.drawImage(image, (x + effectDx) * scale + dx, (y + effectDy)
					* scale + dy, width * scale, length * scale, null);
		}
	}

	/**
	 * get effect bounds
	 */
	public Rectangle getBounds() {
		int rx = x - width / 2;
		int ry = y - length / 2;
		return new Rectangle(rx, ry, width, length);
	}

	/**
	 * constructor
	 * @param caster the spell catser
	 * @param target the spell target
	 * @param icons the icons used for painting
	 */
	public MagicEffect(SpellCaster caster, Warrior target, ImageIcon[] icons) {
		this.target = target;
		this.caster = caster;
		this.x = caster.getX();
		this.y = caster.getY();
		this.xd = target.x;
		this.yd = target.y;
		width = 5;
		length = 5;
		effectDx = (int) (0 - width / 2);
		effectDy = (int) (0 - length / 2);
		this.icons = icons;
	}
	/**
	 * animate the effect (determine next icon used for painting)
	 */
	public void animate() {
		int distx, disty, xStep, yStep;
		double alpha;
		distx = Math.abs(xd - x);
		disty = Math.abs(yd - y);
		if (distx > disty) {
			xStep = (int) ((double) step / (double) (1.0 + (double) disty
					/ (double) distx));
			yStep = step - xStep;
		} else {
			yStep = (int) ((double) step / (double) (1.0 + (double) distx
					/ (double) disty));
			xStep = step - yStep;
		}
		if (xStep > distx) {
			xStep = distx;
		}
		if (yStep > disty) {
			yStep = disty;
		}
		if (x > xd) {
			xStep = -xStep;
		}
		if (y > yd) {
			yStep = -yStep;
		}
		alpha = Math.PI + Math.atan2((double) xStep, (double) yStep);
		doMove(xStep, yStep);
		if (x == xd && y == yd) { //destination reached
			if (target.mustDie()) {
				target.setMustDie(false);
				target.setAction('d');
				target.animationCounter = 1;
			}
			BattleFieldModel.getInstance().removeMagic(this);
			caster.getMagics().remove(this);
		}
		if (image == null) {
			image = (icons[GeometryTools.to8Base(alpha) % icons.length])
					.getImage();
		}
	}
	/**
	 * apply specified move
	 * 
	 * @param xOffset
	 *            x Offset
	 * @param yOffset
	 *            y Offset
	 */
	public void doMove(int xOffset, int yOffset) {
		if (xOffset != 0 || yOffset != 0) {
			Rectangle previousBounds = getBounds();
			x += xOffset;
			y += yOffset;
			firePropertyChange(BOUNDS, previousBounds, getBounds());
		}
	}

}