/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import fb.army.hierarchy.Army;

/**
 * calculate a movement zone in campaign map using a-star algorythm
 * 
 * @author farfpeuf
 * 
 */
public class CampaignMovementZoneFinder extends CampaignPathFinder {

	/**
	 * constructor
	 * 
	 * @param origin
	 *            the zone origin point
	 * @param army
	 *            army for which zone is searched
	 * @param bounds
	 *            map bounds
	 */
	public CampaignMovementZoneFinder(Point origin, Army army, Rectangle bounds) {
		super(origin, null, army, bounds);

	}

	/**
	 * @return true if search must be stopped
	 */
	protected boolean stopSearch() {
		if (super.stopSearch()) {
			return true;
		}
		Node nextNode = (Node) toExplore.get(0);
		return nextNode.getCostFromOrig() > army.getRemainingMove();
	}

	/**
	 * @return the found movement zone as a list of points
	 */
	public List getMovementZone() {
		List zone = new ArrayList();
		Iterator iter = explored.iterator();
		Node node;
		while (iter.hasNext()) {
			node = (Node) iter.next();
			zone.add(node.getPosition());
		}
		return zone;
	}

}
