/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import fb.gui.widgets.FBTextArea;
import fb.main.FB;
import fb.main.Battle;
import fb.util.FBColors;
import fb.util.Resources;

/**
 * displays current player, time and phase and report, help and chat tabs
 */
public class StatePanel extends JPanel {
	/**
	 * panel containing time of day icons
	 */
	protected JPanel timepanel = new JPanel();

	/**
	 * sub-panels
	 */
	protected JLabel timelabel = new JLabel(), phaselabel = new JLabel(),
			playerLabel = new JLabel();

	/**
	 * the tabs panel
	 */
	protected JTabbedPane tabbedPane = new JTabbedPane();

	/**
	 * area containing help text
	 */
	protected FBTextArea helpArea = new FBTextArea(false);

	/**
	 * bring help tab to front
	 * 
	 */
	public void showHelp() {
		tabbedPane.setSelectedIndex(1);
	}

	/**
	 * build all the tabs (report, help and chat)
	 */
	public void buildTabs() {
		tabbedPane.removeAll();
		tabbedPane.addTab(Resources.getString("Report"), MessageArea
				.getInstance());

		tabbedPane.addTab(Resources.getString("Help"), helpArea);
		if (FB.isNet()) {
			tabbedPane.addTab(Resources.getString("Chat"), ChatArea
					.getInstance());
		}

		MessageArea.getInstance().setToolTipText(
				Resources.getString("report.tt"));
		helpArea.setToolTipText(Resources.getString("help.tt"));
	}

	/**
	 * constructor build and assemble panel elements
	 */
	public StatePanel() {

		setLayout(new GridBagLayout());

		timepanel.setLayout(new GridLayout(1, 2));
		timelabel = new JLabel(Resources.getSign("time"
				+ Battle.getInstance().getTime()));

		timepanel.add(timelabel);
		timepanel.add(phaselabel);

		timepanel.setBackground(FBColors.background);
		playerLabel = new JLabel();

		playerLabel.setHorizontalAlignment(0);
		playerLabel.setBackground(FBColors.background);
		playerLabel.setOpaque(true);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		add(timepanel, gbc);
		gbc.gridy++;
		gbc.weighty = 1;
		gbc.fill = GridBagConstraints.BOTH;
		add(playerLabel, gbc);
		gbc.gridx++;
		gbc.gridy = 0;
		gbc.gridheight = 2;
		gbc.weightx = 1;
		add(tabbedPane, gbc);
		timelabel.setToolTipText(Resources.getString("turn.tt"));
		phaselabel.setToolTipText(Resources.getString("phase.tt"));
		playerLabel.setToolTipText(Resources.getString("player.tt"));

	}

	/**
	 * change current phase
	 * 
	 * @param icon
	 *            the new phase icon
	 */
	public void setPhase(ImageIcon icon) {
		phaselabel.setIcon(icon);
	}

	/**
	 * change time of day
	 * 
	 * @param time
	 *            the new time
	 */
	public void setTime(int time) {

		timelabel.setIcon(Resources.getTimeIcon(time - 1));

	}

	/**
	 * change current player
	 * 
	 * @param color
	 *            player color
	 * @param name
	 *            player name
	 * @param ai
	 *            is player human or computer ?
	 * @param handled
	 *            is plyer handled by this computer
	 */
	public void setPlayer(Color color, String name, boolean ai, boolean handled) {
		if (null == name) {
			playerLabel.setText("");
			playerLabel.setBorder(null);
		} else {

			playerLabel.setForeground(color);
			String texte = name;
			if (ai) {
				texte += (" (" + Resources.getString("AI") + ")");
			}
			if (handled && !ai) {
				playerLabel.setBorder(BorderFactory
						.createLineBorder(Color.CYAN));
			} else {
				playerLabel.setBorder(null);
			}
			playerLabel.setText(texte);
		}

	}

	/**
	 * display help message
	 * 
	 * @param mess
	 *            the help message
	 */
	public void setHelp(String mess) {
		helpArea.setText(mess, FBColors.standardText);
	}

}
