/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;

import fb.army.entities.FlyingEntity;
import fb.army.entities.Unit;
import fb.gui.frames.BattleFrame;
import fb.gui.widgets.FBButton;
import fb.util.Resources;

/**
 * panel to choose unit manoeuvres
 * 
 * @author farfpeuf
 * 
 */
public class UnitManoeuvrePanel extends JPanel {
	/**
	 * current unit
	 */
	private Unit unit;

	/**
	 * available manoeuvres buttons
	 */
	private FBButton takeOff, land, left90, right90, leftwheel, rightwheel,
			expend, reduce, surround, reorganize;

	/**
	 * change unit
	 * 
	 * @param u
	 *            the new unit
	 */
	public void setUnit(Unit u) {
		unit = u;
		enableButtons();
	}

	/**
	 * calculate buttons enabling
	 * 
	 */
	public void enableButtons() {
		int canexpend = unit.canExpend();
		int canreduce = unit.canReduce();
		takeOff.setVisible(unit instanceof FlyingEntity);
		land.setVisible(unit instanceof FlyingEntity);
		takeOff.setEnabled(unit instanceof FlyingEntity
				&& ((FlyingEntity) unit).canTakeOff());
		land.setEnabled(unit instanceof FlyingEntity
				&& ((FlyingEntity) unit).canLand());
		expend.setEnabled(canexpend > 0);
		reduce.setEnabled(canreduce > 0);
		left90.setEnabled(unit.canLeft90());
		right90.setEnabled(unit.canRight90());
		leftwheel.setEnabled(unit.canleftwheel());
		rightwheel.setEnabled(unit.canRightWheel());
		surround.setEnabled(unit.canSurround());
		reorganize.setEnabled(unit.canReorganize());
	}

	/**
	 * constructor build and assemble panel elements
	 */
	public UnitManoeuvrePanel() {
		setLayout(new GridLayout(2, 5));
		takeOff = new FBButton(null, "take_off", true);
		land = new FBButton(null, "land", true);
		left90 = new FBButton(null, "90left", true);
		right90 = new FBButton(null, "90right", true);
		leftwheel = new FBButton(null, "leftwheel", true);
		rightwheel = new FBButton(null, "rightwheel", true);
		expend = new FBButton(null, "expend", true);
		reduce = new FBButton(null, "reduce", true);
		surround = new FBButton(null, "surround", true);
		reorganize = new FBButton(null, "reorganize", true);
		left90.setToolTipText(Resources.getString("left90.tt"));
		right90.setToolTipText(Resources.getString("right90.tt"));
		expend.setToolTipText(Resources.getString("expand.tt"));
		leftwheel.setToolTipText(Resources.getString("leftwheel.tt"));
		rightwheel.setToolTipText(Resources.getString("rightwheel.tt"));
		reduce.setToolTipText(Resources.getString("reduce.tt"));
		surround.setToolTipText(Resources.getString("surround.tt"));
		reorganize.setToolTipText(Resources.getString("reorganize.tt"));
		takeOff.setToolTipText(Resources.getString("takeoff.tt"));
		land.setToolTipText(Resources.getString("land.tt"));
		add(left90);
		add(right90);
		add(expend);
		add(surround);
		add(land);
		add(leftwheel);
		add(rightwheel);
		add(reduce);
		add(reorganize);
		add(takeOff);
		setSize(300, 120);
		setOpaque(false);
		validate();
		takeOff.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				((FlyingEntity) unit).takeOff();
				enableButtons();
			}
		});
		land.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				((FlyingEntity) unit).land();
				enableButtons();
			}
		});
		leftwheel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.leftWheel();
				enableButtons();
			}
		});
		rightwheel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.rightWheel();
				enableButtons();
			}
		});
		left90.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.left90();
				enableButtons();
			}
		});
		right90.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.right90();
				enableButtons();
			}
		});
		surround.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.surround();
				enableButtons();
			}
		});
		reorganize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.reorganize();
				enableButtons();
			}
		});
		expend.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				BattleFrame.getInstance().setExpandPanel(unit);
			}
		});
		reduce.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				BattleFrame.getInstance().setReducePanel(unit);
			}
		});
	}
}