/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Component;
import java.awt.Graphics;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

import fb.util.Resources;

/**
 * table cell renderer to display if player is human or artificial intelligence
 * 
 * @author farfpeuf
 * 
 */
public class AIRenderer extends JComponent implements TableCellRenderer {

	/**
	 * checked and uncheked icon
	 */
	private static ImageIcon checked = Resources.getLookNFeelImage("iacheckedbox"),
			unchecked = Resources.getLookNFeelImage("iauncheckedbox");

	/**
	 * current rendered icon
	 */
	private ImageIcon currentIcon = unchecked;

	/**
	 * current value (checked or unchecked)
	 */
	public boolean value = false;

	/**
	 * table cell renderer implementation
	 */
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		setValue(((Boolean) value).booleanValue());
		return this;
	}

	// overidden to remove border
	public void paintBorder(Graphics g) {
	}

	/**
	 * overidden to draw icon
	 */
	public void paintComponent(Graphics g) {
		int min = getWidth();
		if (getHeight() < min) {
			min = getHeight();
		}
		g.drawImage(currentIcon.getImage(), (getWidth() - min) / 2,
				(getHeight() - min) / 2, min, min, null);
	}

	/**
	 * change displayed icon
	 * 
	 * @param icon
	 *            the new icon
	 */
	private void setIcon(ImageIcon icon) {
		currentIcon = icon;
		repaint();
	}

	/**
	 * Returns the value.
	 * 
	 * @return boolean
	 */
	public boolean isValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * 
	 * @param value
	 *            The value to set
	 */
	public void setValue(boolean value) {
		if (this.value != value) {
			this.value = value;
			if (value) {
				setIcon(checked);
			} else {
				setIcon(unchecked);
			}
		}
	}

	/**
	 * Sets the value.
	 * 
	 * @param value
	 *            The value to set
	 */
	public void setValue(Boolean value) {
		setValue(value.booleanValue());
	}

}