/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Component;
import java.awt.FlowLayout;

import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;

import fb.army.entities.Entity;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.army.entities.Unit;
import fb.util.Resources;

/**
 * renderer to display a command
 * 
 * @author farfpeuf
 * 
 */
public class CommandCell extends JPanel implements ListCellRenderer {

	/**
	 * ListCellRenderer implementation
	 */
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {

		Entity e = (Entity) value;

		removeAll();
		setLayout(new FlowLayout());

		JLabel cellname = new JLabel(e.getFullName(), 0);
		String options = "";

		if (e.isUnit()) {
			Unit u = (Unit) e;
			options += ("(" + u.getNumberString() + ")");
			if (u.isUpgraded()) {
				options += " " + Resources.getString("Upgraded");
			}
			if (u.hasChampion()) {
				options += ", " + Resources.getString("Champion");
			}
			if (u.hasStandardBearer()) {
				options += ", " + Resources.getString("Standardbearer");
			}
			if (u.getMusicians() > 0) {
				options += ", " + Resources.getString("Musician");
			}
		}

		if (e.isLeader()) {
			Leader l = (Leader) e;
			options += (Resources.getString("Leadership") + " : " + l
					.getLeadership());
		}

		if (e.isMagic()) {
			SpellCaster m = (SpellCaster) e;
			options += (Resources.getString("Magic") + " : " + m
					.getInitialMagicPoints());
		}

		JLabel celloptions = new JLabel(options, 0);

		JLabel cellcost = new JLabel(Resources.getString("Cost") + " : "
				+ e.getCost(), 0);

		add(cellname);
		add(celloptions);
		add(cellcost);

		if (isSelected) {
			cellname.setBackground(list.getSelectionBackground());
			cellname.setForeground(list.getSelectionForeground());
			celloptions.setBackground(list.getSelectionBackground());
			celloptions.setForeground(list.getSelectionForeground());
			cellcost.setBackground(list.getSelectionBackground());
			cellcost.setForeground(list.getSelectionForeground());

		} else {
			cellname.setBackground(list.getBackground());
			cellname.setForeground(list.getForeground());
			celloptions.setBackground(list.getBackground());
			celloptions.setForeground(list.getForeground());
			cellcost.setBackground(list.getBackground());
			cellcost.setForeground(list.getForeground());

		}

		setEnabled(list.isEnabled());
		setFont(list.getFont());
		return this;
	}
}
