/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.border.Border;

import fb.util.FBColors;
import fb.util.Resources;

/**
 * a fantasy battlefields custom button
 * 
 * @author farfpeuf
 * 
 */
public class FBButton extends JButton implements MouseListener,
		MouseMotionListener {
	/**
	 * button icons
	 */
	protected ImageIcon courant, normal, rollover, pressed, disabled, icon;

	/**
	 * button action listener
	 */
	protected ActionListener actionlistener = null;

	/**
	 * is button enabled ?
	 */
	protected boolean enabled = true;

	/**
	 * is mouse currenlty over the button
	 */
	protected boolean mouseOver = false;

	/**
	 * is button opaque
	 */
	protected boolean opaque = true;

	/**
	 * the button text color
	 */
	private Color textcolor = FBColors.buttonText;

	/**
	 * constructor for buttons with text and icon
	 * 
	 * @param txt
	 *            button text
	 * @param iconName
	 *            button icon name
	 * @param square
	 *            is it a square button ?
	 */
	public FBButton(String txt, String iconName, boolean square) {
		this(txt, iconName, square, true);
	}

	/**
	 * constructor for buttons with text and icon
	 * 
	 * @param txt
	 *            button text
	 * @param iconName
	 *            button icon name
	 * @param square
	 *            is it a square button ?
	 * @param opaque
	 *            is button opaque ?
	 */
	public FBButton(String txt, String iconName, boolean square, boolean opaque) {
		super(txt);
		this.opaque = opaque;
		// setEditable(false);
		if (square) {
			normal = Resources.getLookNFeelImage("sqnormalbouton");
			rollover = Resources.getLookNFeelImage("sqrolloverbouton");
			pressed = Resources.getLookNFeelImage("sqpressedbouton");
			disabled = Resources.getLookNFeelImage("sqdisabledbouton");
		} else if (opaque) {
			normal = Resources.getLookNFeelImage("normalbouton");
			rollover = Resources.getLookNFeelImage("rolloverbouton");
			pressed = Resources.getLookNFeelImage("pressedbouton");
			disabled = Resources.getLookNFeelImage("disabledbouton");
		}

		if (iconName != null) {
			icon = Resources.getButtonIcon(iconName);
		}
		setIcon(normal);
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	/**
	 * constructor for buttons with text only
	 * 
	 * @param txt
	 *            button text
	 */
	public FBButton(String txt) {
		super(txt);
		setToolTipText(txt);
		// setEditable(false);
		normal = Resources.getLookNFeelImage("normalbouton");
		rollover = Resources.getLookNFeelImage("rolloverbouton");
		pressed = Resources.getLookNFeelImage("pressedbouton");
		disabled = Resources.getLookNFeelImage("disabledbouton");

		setIcon(normal);
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	/**
	 * constructor for buttons with text only
	 * 
	 * @param txt
	 *            button text
	 * @param backgroundPathPrefix
	 *            custom background path prefix
	 */
	public FBButton(String txt, String backgroundPathPrefix) {
		super();
		normal = Resources.getLookNFeelImage(backgroundPathPrefix + "normalbouton");
		rollover = Resources.getLookNFeelImage(backgroundPathPrefix + "rolloverbouton");
		pressed = Resources.getLookNFeelImage(backgroundPathPrefix + "pressedbouton");
		disabled = Resources.getLookNFeelImage(backgroundPathPrefix + "disabledbouton");
		setIcon(normal);
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	/**
	 * change button icon
	 * 
	 * @param icon
	 *            the new icon
	 */
	private void setIcon(ImageIcon icon) {
		if (opaque) {
			courant = icon;
			repaint();
		}
	}

	/**
	 * paint the button
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (opaque) {
			g
					.drawImage(courant.getImage(), 0, 0, getWidth(),
							getHeight(), null);
		}

		if (icon != null) {
			g.drawImage(icon.getImage(), 5, 5, getWidth() - 10,
					getHeight() - 10, null);
		}
		if (getText() != null) {
			g.setColor(textcolor);
			java.awt.font.FontRenderContext frc = ((Graphics2D) g)
					.getFontRenderContext();
			java.awt.geom.Rectangle2D r2d = g.getFont().getStringBounds(
					getText(), frc);

			int x = (int) ((getWidth() - r2d.getWidth()) / 2);
			int y = (int) ((getHeight() + r2d.getHeight()) / 2);

			g.drawString(getText(), x, y);
		}

	}

	/**
	 * handle mouse over button
	 */
	public void mouseEntered(MouseEvent event) {
		if (enabled) {
			setIcon(rollover);
		}
		mouseOver = true;
	}

	/**
	 * handle mouse exited button
	 */
	public void mouseExited(MouseEvent event) {
		if (enabled) {
			setIcon(normal);
		}
		mouseOver = false;
	}

	// MouseMotionListener implementaion

	public void mouseMoved(MouseEvent event) {
	}

	public void mouseDragged(MouseEvent event) {
	}

	public void mouseClicked(MouseEvent event) {
	}

	/**
	 * button pressed
	 */
	public void mousePressed(MouseEvent event) {
		if (enabled) {
			setIcon(pressed);
		}
	}

	/**
	 * button released
	 */
	public void mouseReleased(MouseEvent event) {
		if (enabled) {
			setIcon(rollover);
			clic();
		}
	}

	/**
	 * button was clicked
	 * 
	 */
	protected void clic() {
		if (actionlistener != null) {
			actionlistener.actionPerformed(new ActionEvent(this, 0, "clic"));
		}
	}

	// no border
	public void setBorder(Border b) {
	}

	/**
	 * enable or disable button
	 */
	public void setEnabled(boolean enabled) {
		this.enabled = enabled;
		if (!enabled) {
			setIcon(disabled);
			textcolor = Color.lightGray;
		} else if (mouseOver) {
			setIcon(rollover);
			textcolor = FBColors.highlightedText;
		} else {
			setIcon(normal);
			textcolor = FBColors.buttonText;
		}
	}

	/**
	 * set button action listener
	 */
	public void addActionListener(ActionListener al) {
		actionlistener = al;
	}

}
