/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import fb.gui.frames.BattleFrame;
import fb.util.Resources;

/**
 * base class for all game phases
 * 
 * @author farfpeuf
 * 
 */
public abstract class Phase {
	/**
	 * phase waiting for player action ?
	 */
	protected boolean waitingForPlayerAction = false;

	/**
	 * pahse uses an action panel
	 */
	protected boolean withPhasePanel = false;

	/**
	 * phase stopped ?
	 */
	protected boolean stop = false;

	/**
	 * stop the phase
	 * 
	 */
	public void stop() {
		stop = true;
	}

	/**
	 * pause current phase for a determined number of milliseconds
	 * 
	 * @param millis
	 *            number of milliseconds
	 */
	public void sleep(long millis) {
		if (!stop) {
			try {
				Thread.sleep(millis);
			} catch (InterruptedException e) {
				// ignore
			}
		}
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return Resources.getString(getName() + ".tt");
	}

	/**
	 * initialize this phase
	 * 
	 */
	public void init() {
		BattleFrame.getInstance().setPhase(this);
	}

	/**
	 * procced to phase next step
	 * 
	 */
	public abstract void nextStep();

	/**
	 * end this phase
	 * 
	 */
	public abstract void end();

	/**
	 * @return true if phase is finished
	 */
	public abstract boolean finished();

	/**
	 * @return phase name
	 */
	public abstract String getName();

	/**
	 * @return phase icon number
	 */
	public abstract int getIconNumber();
	/**
	 * phase waiting for network action ?
	 */
	public boolean isWaitingForNetAction() {
		return false;
	}

	/**
	 * @return Returns the withPhasePanel.
	 */
	public boolean isWithPhasePanel() {
		return withPhasePanel;
	}

	/**
	 * @param withPhasePanel
	 *            The withPhasePanel to set.
	 */
	public void setWithPhasePanel(boolean withPhasePanel) {
		this.withPhasePanel = withPhasePanel;
	}

	/**
	 * @return Returns the waitForPlayerAction.
	 */
	public boolean isWaitingForPlayerAction() {
		return waitingForPlayerAction;
	}

}
