/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.awt.Point;
import java.util.List;
import java.util.Vector;

import fb.army.entities.Entity;
import fb.army.entities.SpellCaster;
import fb.gui.frames.BattleFrame;
import fb.main.Battle;
import fb.sound.SoundPlayer;
import fb.spells.BasicSpell;
import fb.spells.Resurection;
import fb.spells.TargetSpell;

/**
 * magic phase for human players
 * 
 * @author farfpeuf
 * 
 */
public class PlayerMagicPhase extends InterruptingPhase {
	/**
	 * phase name
	 */
	private static final String name = "magiqphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 5;

	/**
	 * selected spell
	 */
	protected BasicSpell spell;

	/**
	 * current step is spell selection ?
	 */
	private boolean spellSelection = false;

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return null;// no tooltip
	}

	/**
	 * calculate available targets
	 * 
	 */
	public void calculatePotentialTargets() {

		Entity entity;
		clearPotentialTargets();
		Vector entities;
		if (spell instanceof Resurection) {
			entities = Battle.getInstance().getAllEntities();
		} else {
			entities = Battle.getInstance().getActiveEntities();
		}
		for (int j = 0; j < entities.size(); j++) {
			entity = (Entity) (entities.get(j));
			if (entity.isActive() || spell instanceof Resurection) {
				if (isTarget(entity)) {
					addPotentialTarget(entity);
				}
			}
		}
		setActorselection(false);
	}

	/**
	 * initialize actor selection on the battlefield
	 * 
	 */

	public void initActorSelection() {
		super.initActorSelection();
		if (getPotentialActors().size() == 1) {
			setActor((Entity) getPotentialActors().get(0));
			BattleFrame.getInstance().setMagPanel(this);
			spellSelection = true;
			setActorselection(false);
			clearPotentialTargets();
		}
	}

	/**
	 * handle human player action
	 * 
	 */
	public void handlePlayerAction() {
		if (!spellSelection) {
			Entity entity;
			boolean entityselected = false;
			List selectables;
			if (isActorSelection()) {
				selectables = getPotentialActors();
			} else {
				selectables = getPotentialTargets();
			}
			for (int i = 0; i < selectables.size(); i++) {
				entity = (Entity) (selectables.get(i));
				if (entity.getArea().contains(clickedPoint)) {
					entityselected = true;
					selected = entity;
				}
			}
			if (entityselected) {
				BattleFrame.getInstance()
						.displayEntityCharacteristics(selected);
				if (isActorSelection()) {
					setActor(selected);
					initSpellSelection();
				} else {
					((TargetSpell) spell).setTarget(selected);
					BattleFrame.getInstance().getSpellPanel().setTarget(
							selected);
					action();
				}
			} else {
				initActorSelection();
			}
		} else {
			if (spell != null) {
				spellSelection = false;
				BattleFrame.getInstance().getSpellPanel().setSpell(spell);
				BattleFrame.getInstance().setActionPanel(
						BattleFrame.getInstance().getSpellPanel());
				if (spell instanceof TargetSpell) {
					calculatePotentialTargets();
					BattleFrame.getInstance().getSpellPanel()
							.setAvailableTargets(getPotentialTargets());
				}
			} else {
				initActorSelection();
			}
		}
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity e) {
		// return spell.istarget(actor,e);
		return true;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity e) {
		return spell.isTarget(e);
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		if (e.isMagic()) {
			if (((SpellCaster) e).getMagicPoints() > 0) {
				return true;
			}
		}
		return false;
	}

	/**
	 * proceed to phase action
	 * 
	 */
	public void action() {
		SoundPlayer.getInstance().play("magiecast");
		spell.action();
		if (spell instanceof TargetSpell) {
			calculatePotentialTargets();
			BattleFrame.getInstance().getSpellPanel().setAvailableTargets(
					getPotentialTargets());
		}
	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {
	}

	/**
	 * set selected spell
	 * 
	 * @param s
	 *            selected spell
	 */

	public synchronized void setSpell(BasicSpell s) {
		if (waitingForPlayerAction) {
			waitingForPlayerAction = false;
			spell = s;
			spell.setCaster((SpellCaster) getActor());
			clickedPoint = new Point();
			playerPerformedAction();
			Battle.getLoop().resume();

		}
	}

	/**
	 * cast selected spell
	 * 
	 */
	public synchronized void castSpell() {
		action();
	}

	/**
	 * initialize spell selection
	 * 
	 */
	public void initSpellSelection() {
		BattleFrame.getInstance().setMagPanel(this);
		spellSelection = true;
		setActorselection(false);
		clearPotentialTargets();
		spell = null;
	}

	/**
	 * @return Returns the spell.
	 */
	public BasicSpell getSpell() {
		return spell;
	}

}