/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.spells;

import java.util.List;

import fb.army.entities.Druid;
import fb.army.entities.Entity;
import fb.army.entities.Heroic;
import fb.army.entities.Leader;
import fb.army.entities.SpellCaster;
import fb.army.entities.Wizard;
import fb.main.Battle;
import fb.util.D6;

/**
 * Creates a shield against magic around the friendly target
 * 
 * @author farfpeuf
 * 
 */
public class MagicShield extends TargetSpell {

	/**
	 * constructor
	 * 
	 */
	public MagicShield() {
		super();
		name = "magicshield";
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            spell actor
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for specified actor
	 */

	public boolean isTarget(Entity a, Entity e) {
		return !e.isEnnemy(a) && a.lineOfSightTo(e) < 9999
				&& super.isTarget(a, e);
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            spell target
	 * @return true if this entity is a potential target for current spell
	 *         caster
	 */
	public boolean isTarget(Entity e) {
		return !e.isEnnemy(caster) && caster.lineOfSightTo(e) < 9999
				&& super.isTarget(e);
	}

	/**
	 * cast the spell
	 * 
	 * @return true if the spell was successfull
	 */
	public boolean castSpell() {
		if (caster.spendMagicPoints(cost())) {
			return true;
		}
		return false;
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell cost
	 */
	public int cost(SpellCaster caster, Entity target) {
		return (int) (0.999 + caster.lineOfSightTo(target) / 12) + D6.roll()
				+ D6.roll() + D6.roll();

	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell maximum cost
	 */
	public int maxCost(SpellCaster caster, Entity target) {
		return 18 + (int) (caster.lineOfSightTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return spell minimumu cost
	 */
	public int minCost(SpellCaster caster, Entity target) {
		return 3 + (int) (caster.lineOfSightTo(target) / 12);
	}

	/**
	 * 
	 * @param caster
	 *            spell caster
	 * @param target
	 *            spell target
	 * @return average cost
	 */
	public int averageCost(SpellCaster caster, Entity target) {
		return (int) (10.5 + (int) (caster.lineOfSightTo(target) / 12));
	}

	/**
	 * end the spell
	 * 
	 * @throws InterruptedException
	 */
	public void endSpell() throws InterruptedException {
		target.initMagicShield();
		magicSynchronize();
	}

	/**
	 * @param potentialTarget
	 *            the target
	 * @return weight modifiers for artificial intelligence for specified target
	 */
	public int getWeight(Entity potentialTarget) {
		int weight = super.getWeight(potentialTarget);

		if (potentialTarget instanceof Heroic
				|| potentialTarget instanceof Leader
				|| potentialTarget instanceof SpellCaster) {
			weight += 2;
		}
		if (potentialTarget.getTotalLifePoints() == 1) {
			weight += 3;
		}
		return weight;
	}

	/**
	 * @return true if AI player should cast this spell
	 */
	public boolean shouldAICast() {
		if (!super.shouldAICast()) {
			return false;
		}
		List entities = Battle.getInstance().getActiveEntities();
		Entity entity;
		for (int i = 0; i < entities.size(); i++) {// at least an ennemy druid
													// or wizard
			entity = (Entity) entities.get(i);
			if ((entity instanceof Druid || entity instanceof Wizard)
					&& entity.isEnnemy(caster)) {
				return true;
			}
		}
		return false;
	}
}