/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;

import fb.main.Battle;

/**
 * utility class for graphics
 * 
 * @author farfpeuf
 * 
 */
public class GraphicTools {
	/**
	 * draw a circle
	 * 
	 * @param g
	 *            graphics
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 * @param scale
	 *            scale
	 * @param radius
	 *            circle radius
	 * @param x
	 *            center x coordinate
	 * @param y
	 *            center y coordinate
	 */
	public static void drawScaledCircle(Graphics g, int dx, int dy, int scale,
			int radius, int x, int y) {
		Shape circle = new Arc2D.Double(x - radius, y - radius, 2 * radius,
				2 * radius, 0, 360, Arc2D.PIE);
		Shape oldClip = g.getClip();
		g.setClip(getScaledShape(Battle.getInstance().getInterior(), dx, dy,
				scale));
		Area scaledCircle = new Area(getScaledShape(circle, dx, dy, scale));
		((Graphics2D) g).draw(scaledCircle);
		g.setClip(oldClip);
	}

	/**
	 * calculate scaled shape
	 * 
	 * @param shape
	 *            original shape
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 * @param scale
	 *            scale
	 * @return scaled shape
	 */
	public static Shape getScaledShape(Shape shape, int dx, int dy, int scale) {
		Shape scaledShape = AffineTransform.getScaleInstance(scale, scale)
				.createTransformedShape(shape);
		scaledShape = AffineTransform.getTranslateInstance(dx, dy)
				.createTransformedShape(scaledShape);
		return scaledShape;
	}

	/**
	 * @return a standard gridbag layout constraint
	 */
	public static GridBagConstraints createStandardConstraint() {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.insets = new Insets(5, 5, 5, 5);
		return gbc;
	}

}
