/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * a tool to help creation of hexagonal tile images for campaign maps
 * 
 * @author farfpeuf
 * 
 */
public class HexagonalTileBuilder extends JFrame {

	/**
	 * hexagonal canvas
	 */
	private HexagonalCanvas canvas;

	/**
	 * button to change color
	 */
	private JButton colorButton;

	/**
	 * tile width field
	 */
	private JTextField widthField;

	/**
	 * tile height field
	 */
	private JTextField heightField;

	/**
	 * paint brush size field
	 */
	private JTextField brushSizeField;

	/**
	 * build, assemble components and register event listeners
	 * 
	 */
	public void init() {

		GridBagConstraints c = new GridBagConstraints();
		getContentPane().setLayout(new GridBagLayout());
		c.gridx = 0;
		c.gridy = 0;
		getContentPane().add(new JLabel("Width"), c);
		widthField = new JTextField("72", 5);
		c.gridx++;
		getContentPane().add(widthField, c);
		c.gridx++;
		getContentPane().add(new JLabel("Height"), c);
		heightField = new JTextField("62", 5);
		c.gridx++;
		getContentPane().add(heightField, c);
		c.gridx++;
		getContentPane().add(new JLabel("Brush size"), c);
		brushSizeField = new JTextField("1", 5);
		c.gridx++;
		getContentPane().add(brushSizeField, c);

		c.gridy++;
		c.gridx = 0;
		JButton load = new JButton("Load");
		getContentPane().add(load, c);

		JButton wrap = new JButton("Wrap");
		c.gridx++;
		getContentPane().add(wrap, c);

		JCheckBox preview = new JCheckBox("Preview");
		c.gridx++;
		getContentPane().add(preview, c);

		JButton zoomIn = new JButton("Zoom in");
		c.gridx++;
		getContentPane().add(zoomIn, c);

		JButton zoomOut = new JButton("Zoom out");
		c.gridx++;
		getContentPane().add(zoomOut, c);

		colorButton = new JButton();
		colorButton.setBackground(Color.BLACK);
		c.gridx++;
		getContentPane().add(colorButton, c);

		c.gridy++;
		canvas = new HexagonalCanvas() {

			public void setPaintingColor(Color paintingColor) {
				super.setPaintingColor(paintingColor);
				colorButton.setBackground(paintingColor);
			}

		};
		c.gridwidth = 6;
		c.gridx = 0;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.CENTER;
		c.weightx = 1.0;
		c.weighty = 1.0;
		getContentPane().add(canvas, c);
		validate();

		load.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadImage();
			}
		});

		wrap.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				canvas.wrapImage();
			}
		});

		zoomIn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				canvas.zoomIn();
			}
		});

		zoomOut.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				canvas.zoomOut();
			}
		});

		preview.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				canvas.switchPreview();
			}
		});

		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeColor();
			}
		});

		widthField.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ev) {
				try {
					int width = Integer.parseInt(((JTextField) ev.getSource())
							.getText());
					canvas.setTileWidth(width);
				} catch (NumberFormatException e) {
					// ignore
				}
			}
		});
		widthField.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent ev) {
				try {
					int width = Integer.parseInt(((JTextField) ev.getSource())
							.getText());
					canvas.setTileWidth(width);
				} catch (NumberFormatException e) {
					// ignore
				}
			}
		});

		heightField.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ev) {
				try {
					int height = Integer.parseInt(((JTextField) ev.getSource())
							.getText());
					canvas.setTileHeight(height);
				} catch (NumberFormatException e) {
					// ignore
				}
			}
		});
		heightField.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent ev) {
				int height = Integer.parseInt(((JTextField) ev.getSource())
						.getText());
				canvas.setTileHeight(height);
			}
		});

		brushSizeField.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ev) {
				try {
					int size = Integer.parseInt(((JTextField) ev.getSource())
							.getText());
					canvas.setBrushSize(size);
				} catch (NumberFormatException e) {
					// ignore
				}
			}
		});

		brushSizeField.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent ev) {
				int size = Integer.parseInt(((JTextField) ev.getSource())
						.getText());
				canvas.setBrushSize(size);
			}
		});

	}

	/**
	 * change paintbrush color
	 * 
	 */
	public void changeColor() {
		Color newColor = JColorChooser.showDialog(this, null, colorButton
				.getBackground());
		if (newColor != null) {
			canvas.setPaintingColor(newColor);
		}
	}

	/**
	 * load a tile image
	 * 
	 */
	public void loadImage() {
		JFileChooser chooser = new JFileChooser();
		int returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile();
			canvas.loadImage(file);
		}
	}

	/**
	 * program main method build window and show it
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		HexagonalTileBuilder frame = new HexagonalTileBuilder();
		frame.init();
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setBounds(500, 500, 0, 0);
		frame.show();
	}

}

/**
 * tilme builder canvas
 * 
 * @author farfpeuf
 * 
 */
class HexagonalCanvas extends JPanel implements DragSourceListener,
		DragGestureListener, DropTargetListener, ClipboardOwner {
	/**
	 * hexagonal tile width
	 */
	private int tileWidth = 72;

	/**
	 * hexagonal tile height
	 */
	private int tileHeight = 62;

	/**
	 * paint brush size
	 */
	private int brushSize = 1;

	/**
	 * canvas scale
	 */
	private double scale = 1;

	/**
	 * preview stiched tiles
	 */
	private boolean preview = false;

	/**
	 * current painting color
	 */
	private Color paintingColor = Color.BLACK;

	/**
	 * tile image
	 */
	private Image tileImage;

	/**
	 * tile file
	 */
	private File file = new File("wrapped.png");

	/**
	 * last pressed mouse button
	 */
	private int lastPressedButton = -1;

	/**
	 * constructor register event listeners
	 */
	public HexagonalCanvas() {
		super();

		dragSource.createDefaultDragGestureRecognizer(this,
				DnDConstants.ACTION_MOVE, this);

		_dropTarget = new DropTarget(this, this);

		addMouseMotionListener(new MouseMotionAdapter() {
			public void mouseDragged(MouseEvent e) {
				if (lastPressedButton == MouseEvent.BUTTON1) {
					changeColor(e.getPoint());
				}
			} // mouse clicked
		});

		addMouseListener(new MouseAdapter() {

			public void mousePressed(MouseEvent e) {
				lastPressedButton = e.getButton();
			}

			public void mouseReleased(MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1) {
					changeColor(e.getPoint());
				} else {
					getColor(e.getPoint());
				}
			} // mouse clicked
		});
	}

	/**
	 * change pixel color
	 * 
	 * @param p
	 *            pixel coordinates
	 */
	public void changeColor(Point p) {
		p.x = (int) (p.x / scale);
		p.y = (int) (p.y / scale);
		if (!preview && tileImage != null && p.x >= 0 && p.x < tileWidth
				&& p.y >= 0 && p.y < tileHeight
				&& tileImage instanceof BufferedImage) {

			int[] rgbArray = new int[brushSize * brushSize];
			for (int i = 0; i < brushSize * brushSize; i++) {
				rgbArray[i] = paintingColor.getRGB();
			}
			((BufferedImage) tileImage).setRGB(p.x - brushSize / 2, p.y
					- brushSize / 2, brushSize, brushSize, rgbArray, 0, 1);
		}
		repaint();
	}

	/**
	 * get pixel color
	 * 
	 * @param p
	 *            pixel coordinates
	 */
	public void getColor(Point p) {
		p.x = (int) (p.x / scale);
		p.y = (int) (p.y / scale);
		if (!preview && tileImage != null && p.x >= 0 && p.x < tileWidth
				&& p.y >= 0 && p.y < tileHeight
				&& tileImage instanceof BufferedImage) {
			setPaintingColor(new Color(((BufferedImage) tileImage).getRGB(p.x,
					p.y)));

		}
	}

	/**
	 * wrap tile image (rotate each of the 6 triangles composing the hexagon by
	 * 120 degrees)
	 * 
	 */
	public void wrapImage() {
		Image sourceImage = tileImage;

		tileImage = this.createImage(tileWidth, tileHeight);
		Graphics2D g = (Graphics2D) tileImage.getGraphics();
		Shape oldClip = g.getClip();

		int npoints = 4;
		int[] xpoints = new int[npoints];
		int[] ypoints = new int[npoints];

		xpoints[0] = tileWidth / 2;
		xpoints[1] = 0;
		xpoints[2] = tileWidth / 4;
		xpoints[3] = tileWidth * 3 / 4;

		ypoints[0] = tileHeight / 2;
		ypoints[1] = tileHeight / 2;
		ypoints[2] = 0;
		ypoints[3] = 0;

		Polygon clip = new Polygon(xpoints, ypoints, npoints);
		g.setClip(clip);

		if (sourceImage == null) {
			g.setColor(Color.WHITE);
			g.fillPolygon(clip);
		} else {
			g.drawImage(sourceImage, -tileWidth / 4, -tileHeight / 2,
					tileWidth, tileHeight, this);
		}
		xpoints[1] = tileWidth * 3 / 4;
		xpoints[2] = tileWidth;
		xpoints[3] = tileWidth * 3 / 4;

		ypoints[1] = 0;
		ypoints[2] = tileHeight / 2;
		ypoints[3] = tileHeight;

		clip = new Polygon(xpoints, ypoints, npoints);
		g.setClip(clip);
		if (sourceImage == null) {
			g.setColor(Color.blue);
			g.fillPolygon(clip);
		} else {
			g.drawImage(sourceImage, tileWidth / 2, 0, tileWidth, tileHeight,
					this);
		}

		xpoints[1] = tileWidth * 3 / 4;
		xpoints[2] = tileWidth / 4;
		xpoints[3] = 0;

		ypoints[1] = tileHeight;
		ypoints[2] = tileHeight;
		ypoints[3] = tileHeight / 2;

		clip = new Polygon(xpoints, ypoints, npoints);
		g.setClip(clip);

		if (sourceImage == null) {
			g.setColor(Color.black);
			g.fillPolygon(clip);
		} else {
			g.drawImage(sourceImage, -tileWidth / 4, tileHeight / 2, tileWidth,
					tileHeight, this);
		}

		g.setClip(oldClip);

		try {
			FileOutputStream out = new FileOutputStream(file);

			BufferedImage bufferedImage = new BufferedImage(tileWidth,
					tileHeight, BufferedImage.TYPE_INT_BGR);

			bufferedImage.createGraphics().drawImage(tileImage, 0, 0, this);
			ImageIO.write(bufferedImage, "png", out);

			// new GifEncoder(hxImage, out, true).encode();
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		repaint();

	}

	/**
	 * switch preview boolean value
	 * 
	 */
	public void switchPreview() {
		this.preview = !preview;
		repaint();
	}

	/**
	 * change tile height
	 * 
	 * @param hxHeight
	 *            new tile height
	 */
	public void setTileHeight(int hxHeight) {
		this.tileHeight = hxHeight;
		repaint();

	}

	/**
	 * change tile width
	 * 
	 * @param hxWidth
	 *            new tile width
	 */
	public void setTileWidth(int hxWidth) {
		this.tileWidth = hxWidth;
		repaint();
	}

	/**
	 * get hexagonal clip
	 * 
	 * @param dx
	 *            x offset
	 * @param dy
	 *            y offset
	 * @return the clip polygon
	 */
	private Polygon getClip(int dx, int dy) {
		int npoints = 6;
		int[] xpoints = new int[npoints];
		int[] ypoints = new int[npoints];

		xpoints[0] = dx + tileWidth;
		xpoints[1] = dx + tileWidth * 3 / 4;
		xpoints[2] = dx + tileWidth / 4;
		xpoints[3] = dx + 0;
		xpoints[4] = dx + tileWidth / 4;
		xpoints[5] = dx + tileWidth * 3 / 4;

		ypoints[0] = dy + tileHeight / 2;
		ypoints[1] = dy + 0;
		ypoints[2] = dy + 0;
		ypoints[3] = dy + tileHeight / 2;
		ypoints[4] = dy + tileHeight;
		ypoints[5] = dy + tileHeight;

		Polygon clip = new Polygon(xpoints, ypoints, npoints);
		return clip;
	}

	/**
	 * paint the canvas
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		((Graphics2D) g).scale(scale, scale);
		if (tileImage != null) {
			Dimension d = getSize();
			if (preview) {
				int mapWidth = d.width / tileWidth;
				int mapHeight = d.height / tileHeight;
				Shape oldClip = g.getClip();

				for (int i = 0; i < mapWidth; i++) {
					for (int j = 0; j < mapHeight; j++) {
						int xOffset = tileWidth * 3 / 4;
						int yOffset = tileHeight / 2;
						int x = tileWidth / 2 + xOffset * i;
						int y = j * 2 * yOffset;
						if (i % 2 == 1) {
							y += yOffset;
						}
						g.setClip(getClip(x, y));
						g.drawImage(tileImage, x, y, this);

					}

				}

				g.setClip(oldClip);
			} else {

				g.drawImage(tileImage, 0, 0, this);
			}
		} else {
			wrapImage();
		}
	}

	/**
	 * load a tile image
	 * 
	 * @param source
	 *            the image file
	 */
	public void loadImage(File source) {
		this.file = new File(source.getPath().substring(0,
				source.getPath().length() - 4)
				+ "_wrapped.png");
		this.tileImage = (new ImageIcon(source.getPath())).getImage();
		repaint();
	}

	/**
	 * change painting color
	 * 
	 * @param paintingColor
	 *            new painting color
	 */
	public void setPaintingColor(Color paintingColor) {
		this.paintingColor = paintingColor;
	}

	/**
	 * zomm in canvas
	 * 
	 */
	public void zoomIn() {
		if (scale > 1) {
			scale++;
		} else {
			scale *= 2;
		}
		repaint();
	}

	/**
	 * zoom out canvas
	 * 
	 */
	public void zoomOut() {
		if (scale > 1) {
			scale--;
		} else {
			scale /= 2;
		}
		repaint();
	}

	/**
	 * change paint brush size
	 * 
	 * @param brushSize
	 *            new paint brush size
	 */
	public void setBrushSize(int brushSize) {
		this.brushSize = brushSize;
	}

	// ------------------------------------drag---------------

	/**
	 * the drag source
	 */
	private DragSource dragSource = new DragSource();

	/**
	 * mnage drag gesture
	 */
	public void dragGestureRecognized(DragGestureEvent event) {

		if (tileImage != null && lastPressedButton != MouseEvent.BUTTON1) {

			Transferable transferable = new Transferable() {

				public Object getTransferData(DataFlavor flavor)
						throws UnsupportedFlavorException, IOException {

					if (flavor.equals(DataFlavor.imageFlavor)) {
						return tileImage;
					} else {
						List files = new ArrayList();
						files.add(file);
						return files;
					}
				}

				public DataFlavor[] getTransferDataFlavors() {
					return new DataFlavor[] { DataFlavor.imageFlavor,
							DataFlavor.javaFileListFlavor };
				}

				public boolean isDataFlavorSupported(DataFlavor flavor) {
					return flavor.equals(DataFlavor.imageFlavor)
							|| flavor.equals(DataFlavor.javaFileListFlavor);
				}

			};
			Clipboard clipboard = getToolkit().getSystemClipboard();
			clipboard.setContents(transferable, this);
			dragSource.startDrag(event, DragSource.DefaultCopyDrop,
					transferable, this);
		}

	}

	// -------------------------------drag implementation
	public void dragDropEnd(DragSourceDropEvent event) {
	}

	public void dragEnter(DragSourceDragEvent event) {
	}

	public void dragExit(DragSourceEvent event) {
	}

	public void dragOver(DragSourceDragEvent event) {
	}

	public void dropActionChanged(DragSourceDragEvent event) {
	}

	public void lostOwnership(Clipboard cb, Transferable tr) {
	}

	// -----------------------drop

	/**
	 * the drop target
	 */
	private DropTarget _dropTarget;

	/**
	 * manage drag enter event
	 */
	public void dragEnter(DropTargetDragEvent event) {
		event.acceptDrag(DnDConstants.ACTION_MOVE);
	}

	/**
	 * manage drop
	 */
	public void drop(DropTargetDropEvent event) {
		try {

			Transferable transferable = event.getTransferable();
			/*
			 * DataFlavor[] flav = transferable.getTransferDataFlavors(); for
			 * (int i = 0; i < flav.length; i++) { System.out.println(flav[i]); }
			 */
			if (transferable.isDataFlavorSupported(DataFlavor.imageFlavor)) {
				event.acceptDrop(DnDConstants.ACTION_MOVE);
				tileImage = (Image) transferable
						.getTransferData(DataFlavor.imageFlavor);
				event.getDropTargetContext().dropComplete(true);
				repaint();
			} else if (transferable
					.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
				event.acceptDrop(DnDConstants.ACTION_MOVE);
				List files = (List) transferable
						.getTransferData(DataFlavor.javaFileListFlavor);

				if (!files.isEmpty()) {
					loadImage((File) files.get(0));
				}
				event.getDropTargetContext().dropComplete(true);
			} else {
				event.rejectDrop();
			}
		} catch (IOException exception) {
			event.rejectDrop();
		} catch (UnsupportedFlavorException ufException) {
			event.rejectDrop();
		}
	}

	// -------------------------------drop implementation
	public void dragExit(DropTargetEvent event) {
	}

	public void dragOver(DropTargetDragEvent event) {
	}

	public void dropActionChanged(DropTargetDragEvent event) {
	}

}