/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.util;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImageFilter;
import java.awt.image.BufferedImageOp;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ImageProducer;

import javax.swing.ImageIcon;

/**
 * utility class for images manipulations
 * 
 * @author farfpeuf
 * 
 */
public class ImageTools {
	/**
	 * flip an image
	 * 
	 * @param sourceIcon
	 *            source image
	 * @param leftRight
	 *            flip from left to right ?
	 * @param upDown
	 *            flip from up to down ?
	 * @return flipped image
	 */
	public static Image flip(ImageIcon sourceIcon, boolean leftRight,
			boolean upDown) {

		if (!leftRight && !upDown) {
			return sourceIcon.getImage();
		}

		Image resultImage;

		ImageFilter filter;
		ImageProducer producer;
		int width = sourceIcon.getIconWidth();
		int height = sourceIcon.getIconHeight();

		AffineTransform flip = new AffineTransform(leftRight ? -1 : 1, 0, 0,
				upDown ? -1 : 1, leftRight ? width : 0, upDown ? height : 0);

		BufferedImageOp op = new AffineTransformOp(flip,
				AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
		filter = new BufferedImageFilter(op);
		producer = new FilteredImageSource(sourceIcon.getImage().getSource(),
				filter);
		resultImage = Toolkit.getDefaultToolkit().createImage(producer);

		return resultImage;

	}

	/**
	 * rotate an image
	 * 
	 * @param sourceIcon
	 *            dource image
	 * @param theta
	 *            rotation angle
	 * @return rotated image
	 */
	public static Image rotate(ImageIcon sourceIcon, double theta) {

		if (theta == 0) {
			return sourceIcon.getImage();
		}

		Image resultImage;

		ImageFilter filter;
		ImageProducer producer;
		int width = sourceIcon.getIconWidth();
		int height = sourceIcon.getIconHeight();

		AffineTransform rotation = AffineTransform.getRotateInstance(theta,
				width / 2, height / 2);

		BufferedImageOp op = new AffineTransformOp(rotation,
				AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
		filter = new BufferedImageFilter(op);
		producer = new FilteredImageSource(sourceIcon.getImage().getSource(),
				filter);
		resultImage = Toolkit.getDefaultToolkit().createImage(producer);

		return resultImage;

	}

}
